/***************************************************************************
                          mhs_cmd.c  -  description
                             -------------------
    begin             : 12.03.2012
    last modified     : 02.01.2016    
    copyright         : (C) 2012 - 2016 by MHS-Elektronik GmbH & Co. KG, Germany
    author            : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************/

/**
    Library to talk to Tiny-CAN devices. You find the latest versions at
       http://www.tiny-can.com/
**/
#include "global.h"
#include "util.h"
#include "errors.h"
#include "tcan_drv.h"
#ifdef MAIN_THREAD_USE_FILE_EVENT
#include "mhs_file_event.h"
#endif
#include "can_core.h" // can_main_get_context()
#include "mhs_cmd.h"


void mhs_cmd_create(TMhsCmd *cmd)
{
if (!cmd)
  return;
pthread_mutex_init(&cmd->Mutex, NULL);
pthread_mutex_init(&cmd->WaitMutex, NULL);
pthread_cond_init(&cmd->Cond, NULL);
}


void mhs_cmd_destroy(TMhsCmd *cmd)
{
if (!cmd)
  return;
pthread_mutex_destroy(&cmd->Mutex);
pthread_mutex_destroy(&cmd->WaitMutex);
pthread_cond_destroy(&cmd->Cond);
}


/***************************************************************/
/* ExecuteCommand                                              */
/***************************************************************/
int32_t mhs_cmd_execute(TCanDevice *dev, uint32_t command, void *param, uint32_t param_size, int32_t timeout)
{
struct timespec tabs;
TMhsObjContext *context, *main_context;
TMhsCmd *cmd;
uint32_t wait;
int32_t res;

if (!dev)
  return(-1);
cmd = &dev->Cmd;
context = dev->Context;
main_context = can_main_get_context();
if (!dev->MainThread)
  return(0);
if ((!dev->MainThread->Thread) || (!dev->MainThread->Run))
  return(0);
if (timeout > -1)
  wait = timeout;
else
  {  
  if (mhs_value_get_status("ExecuteCommandTimeout", context) & MHS_VAL_FLAG_WRITE)
    wait = mhs_value_get_as_ulong("ExecuteCommandTimeout", ACCESS_INIT, context);
  else
    wait = mhs_value_get_as_ulong("ExecuteCommandTimeout", ACCESS_INIT, main_context);
  }
mhs_calc_abs_timeout(&tabs, wait);
pthread_mutex_lock(&cmd->Mutex);
pthread_mutex_lock(&cmd->WaitMutex);
cmd->Command = command;
cmd->ParamCount = param_size;
cmd->Param = param;
#ifdef MAIN_THREAD_USE_FILE_EVENT
(void)set_file_event(dev->MainThreadEvent, MAIN_CMD_EVENT);
#else
mhs_event_set((TMhsEvent *)dev->MainThread, MAIN_CMD_EVENT);
#endif
if (wait)
  {
  if (pthread_cond_timedwait(&cmd->Cond, &cmd->WaitMutex, &tabs) == 0)
    res = cmd->Result;
  else
    res = ERR_MAIN_THREAD_IS_BUSY;
  }
else
  {
  if (pthread_cond_wait(&cmd->Cond, &cmd->WaitMutex) == 0)
    res = cmd->Result;
  else
    res = -1;
  }  
pthread_mutex_unlock(&cmd->WaitMutex);
pthread_mutex_unlock(&cmd->Mutex);
return(res);
}


void mhs_cmd_finish(TCanDevice *dev, int32_t result)
{
if (!dev)
  return;
pthread_mutex_lock(&dev->Cmd.WaitMutex);
dev->Cmd.Result = result;
pthread_cond_signal(&dev->Cmd.Cond);
pthread_mutex_unlock(&dev->Cmd.WaitMutex);  
}
