/***************************************************************************
                            usb_hlp.c  -  description
                             -------------------
    begin             : 03.11.2010
    copyright         : (C) 2010 by MHS-Elektronik GmbH & Co. KG, Germany
    author            : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************/

/**
    Library to talk to Tiny-CAN devices. You find the latest versions at
       http://www.tiny-can.com/
**/
#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "util.h"
#include "sysfs.h"
/*#ifdef ENABLE_LOG_SUPPORT
#include "log.h"
#endif*/
#include "tcan_drv.h"
//#include "usb_hlp.h"
#include "usb_scan.h"

//struct TTCanUsbList *TCanUsbList;

static const char USB_SERIAL_DEVICES_PATH[] = {"bus/usb-serial/devices"};


void UsbDevListDestroy(TUsbDevList **list)
{
TUsbDevList *ptr, *l;

if (!list)
  return;
l = *list;
while (l)
  {
  ptr = l->Next;
  safe_free(l->DeviceName);
  safe_free(l->Manufacturer);
  safe_free(l->Product);
  safe_free(l->SerialNumber);
  mhs_free(l);
  l = ptr;
  }
*list = NULL;
}


static TUsbDevList *UsbDevAdd(TUsbDevList **list, char *device_name, unsigned short id_product,
                            unsigned short id_vendor, unsigned short bcd_device,
                            char *manufacturer, char *product, char *serial)
{
TUsbDevList *ptr, *neu;

if (!list)
  return(NULL);
neu = (TUsbDevList *)mhs_calloc(1, sizeof(TUsbDevList));
if (!neu)
  return(NULL);        // Nicht gengend Arbetsspeicher
neu->Next = NULL;
neu->idProduct = id_product;
neu->idVendor = id_vendor;
neu->bcdDevice = bcd_device;
neu->Manufacturer = mhs_strdup(manufacturer);
neu->Product = mhs_strdup(product);
neu->SerialNumber = mhs_strdup(serial);
neu->DeviceName = mhs_strdup(device_name);
neu->User = 0;
ptr = *list;
if (!ptr)
  // Liste ist leer
  *list = neu;
else
  { // Neues Element anhngen
  while (ptr->Next != NULL) ptr = ptr->Next;
    ptr->Next = neu;
  }
return(neu);
}


TUsbDevList *UsbScan(void)
{
//int32_t num_devs;
int32_t err;
char *path, *link, *s, *devicename;
DIR *dir;
struct dirent *entry;
char description[MAX_LINE_SIZE];
char serial[MAX_LINE_SIZE];
char manufacturer[MAX_LINE_SIZE];
unsigned short id_product, id_vendor, bcd_device;
uint32_t interface_num, interface_count, save_device;
TUsbDevList *list;

id_product = 0;
id_vendor = 0;
bcd_device = 0;
list = NULL;
devicename = NULL;
link = NULL;
if (!(s = sysfs_get_mnt_path()))
  return(NULL);
path = mhs_strconcat(s, "/", USB_SERIAL_DEVICES_PATH, NULL);
safe_free(s);
if (!(dir = opendir(path)))
  {
  safe_free(path);
  return(NULL);
  }
//num_devs = 0;
while ((entry = readdir(dir)) != NULL)
  {
  save_device = 1;
  interface_count = 0;
  // . u. .. berspringen
  if (entry->d_name[0] == '.')
    continue;
  s = mhs_strconcat(path, "/", entry->d_name, NULL);
  link = mhs_get_link(s);
  safe_free(s);
  if (!link)
    continue;
  // **** Device Name
  devicename = mhs_strconcat("/dev/", entry->d_name, NULL);
  if ((s = strrchr(link, '/')))
    *s = '\0';
  interface_num = mhs_sys_read_as_ulong(link, "bInterfaceNumber", 0, &err);
  if (!err)
    {
    if ((s = strrchr(link, '/')))
      *s = '\0';
    interface_count = mhs_sys_read_as_ulong(link, "bNumInterfaces", 0, &err);
    }
  // **** Description
  if (!err)
    {
    if (mhs_sys_read_value(link, "product", description, MAX_LINE_SIZE) <= 0)
      err = -1;
    }
  if (!err)
    {
    // **** Serial
    if (mhs_sys_read_value(link, "serial", serial, MAX_LINE_SIZE) <= 0)
      err = -1;
    }
  if (!err)
    {
    if (mhs_sys_read_value(link, "manufacturer", manufacturer, MAX_LINE_SIZE) <= 0)
      err = -1;
    }
  if (!err)
    {
    id_product = (unsigned short) mhs_sys_read_as_ulong(link, "idProduct", 16, &err);
    if (!err)
      {
      id_vendor = (unsigned short) mhs_sys_read_as_ulong(link, "idVendor", 16, &err);
      if (!err)
        bcd_device = (unsigned short) mhs_sys_read_as_ulong(link, "bcdDevice", 16, &err);
      }
    }
  if ((interface_count > 1) && (interface_num > 0))  // <*> Eigentlich so nicht ganz schn,
    save_device = 0;                                 // es kann nur das der 1. Port angesprochen werden.
/*    if (interface_count > 1)
      {
      switch (interface_num)
        {
        case 0: {
                strcat(serial, "A");
                strcat(manufacturer, " A");
                break;
                }
        case 1: {
                strcat(serial, "B");
                strcat(manufacturer, " B");
                break;
                }
        case 2: {
                strcat(serial, "C");
                strcat(manufacturer, " C");
                break;
                }
        case 3: {
                strcat(serial, "D");
                strcat(manufacturer, " D");
                break;
                }
        }
      } */
  if ((!err) && (save_device))
    (void)UsbDevAdd(&list, devicename, id_product, id_vendor, bcd_device, manufacturer, description, serial);
  safe_free(devicename);
  safe_free(link);
  }
closedir(dir);
safe_free(link);
safe_free(path);
safe_free(devicename);
return(list);
}
