#ifndef __LOG_H__
#define __LOG_H__

#include "global.h"
#include "can_types.h"
#include "drv_types.h"

#ifdef __cplusplus
  extern "C" {
#endif

#define LOG_FUNCTION_ERROR(function, res) do {  \
  if ((res < 0) && (LogEnable)) \
    LogPrintf(LOG_ERROR , 0, "[ERROR] function \"%s\" returns with error: %d", function, res); \
  } while(0)

#define LOG_API(function, res) do { \
  if (LogEnable) \
    { \
    if (res < 0) \
      LogPrintf(LOG_ERROR | LOG_API_CALL, 0, "API-Call: %s, result: %s", function, GetErrorString(res)); \
    else \
      LogPrintf(LOG_API_CALL, 0, "API-Call: %s, result: Ok", function); \
    } \
  } while(0)

#define LOG_API_VOID(function) do { \
  if (LogEnable) \
    LogPrintf(LOG_API_CALL, 0, "API-Call: %s", function); \
  } while(0)

#define LOG_API_ENTER(function) do { \
  if (LogEnable) \
    LogPrintf(LOG_API_CALL, 0, "API-Call Enter: %s", function); \
  } while(0)


#define LOG_API_ENTER_IDX(function, idx) do { \
  if (LogEnable) \
    LogPrintf(LOG_API_CALL, 0, "API-Call Enter: %s, index: %#08X", function, idx); \
  } while(0)


#define LOG_API_EXIT_COUNT(function, res) do { \
  if (LogEnable) \
    { \
    if (res < 0) \
      LogPrintf(LOG_ERROR | LOG_API_CALL, 0, "API-Call Exit: %s, result: %s", function, GetErrorString(res)); \
    else \
      LogPrintf(LOG_API_CALL, 0, "API-Call Exit: %s, count: %d", function, res); \
    } \
  } while(0)

#define LOG_API_EXIT_VALUE(function, res, value) do { \
  if (LogEnable) \
    { \
    if (res < 0) \
      LogPrintf(LOG_ERROR | LOG_API_CALL, 0, "API-Call Exit: %s, result: %s", function, GetErrorString(res)); \
    else \
      LogPrintf(LOG_API_CALL, 0, "API-Call Exit: %s, value: %u", function, value); \
    } \
  } while(0)

#define LOG_API_EXIT_STR_VALUE(function, res, value) do { \
  if (LogEnable) \
    { \
    if (res < 0) \
      LogPrintf(LOG_ERROR | LOG_API_CALL, 0, "API-Call Exit: %s, result: %s", function, GetErrorString(res)); \
    else \
      LogPrintf(LOG_API_CALL, 0, "API-Call Exit: %s, value: %s", function, value); \
    } \
  } while(0)

#define LOG_API_ENTER_STR_LIST(function, desc, strlist) do { \
  if (LogEnable) \
    { \
    LogPrintf(LOG_API_CALL, LOG_LOCK, "API-Call Enter: %s, %s", function, desc); \
    LogStringList(LOG_API_CALL, LOG_UNLOCK, strlist); \
    } \
 } while(0)


#define LOG_API_EXIT(function, res) do { \
  if (LogEnable) \
    { \
    if (res < 0) \
      LogPrintf(LOG_ERROR | LOG_API_CALL, 0, "API-Call Exit: %s, result: %s", function, GetErrorString(res)); \
    else \
      LogPrintf(LOG_API_CALL, 0, "API-Call Exit: %s, result: Ok", function); \
    } \
  } while(0)


#define LOG_API_STR_LIST(name, list) do { \
  if (LogEnable) \
    { \
    LogPrintf(LOG_API_CALL, LOG_LOCK, "%s", name); \
    LogStringList(LOG_API_CALL, LOG_UNLOCK, list); \
    } \
  } while(0)


#ifdef ENABLE_LOG_SUPPORT
extern uint32_t LogEnable ATTRIBUTE_INTERNAL;
#else
#define LogEnable 0
#endif

#define LOG_LOCK    0x0001
#define LOG_PASS    0x0002
#define LOG_UNLOCK  0x0004


void LogInit(void) ATTRIBUTE_INTERNAL;
int32_t LogCreate(void) ATTRIBUTE_INTERNAL;
void LogDestroy(void) ATTRIBUTE_INTERNAL;
int32_t LogOpen(char *filename, uint32_t flags) ATTRIBUTE_INTERNAL;
void LogClose(void) ATTRIBUTE_INTERNAL;
//void LogSet(uint32_t log_flags) ATTRIBUTE_INTERNAL;
void LogPrintf(uint32_t log_type, uint32_t log_flags, const char *format_str, ...) ATTRIBUTE_INTERNAL;
void LogCanMessages(uint32_t log_type, uint32_t log_flags, struct TCanMsg *msg, int32_t count) ATTRIBUTE_INTERNAL;
void LogCanMessage(uint32_t log_type, struct TCanMsg *msg, const char *format_str, ...) ATTRIBUTE_INTERNAL;
void LogStringList(uint32_t log_type, uint32_t log_flags, char *str_list) ATTRIBUTE_INTERNAL;
void LogSysError(char *file_name, char *func_name, int32_t error_no) ATTRIBUTE_INTERNAL;
void LogHexDump(uint32_t log_type, uint32_t log_flags, char *data, int32_t count) ATTRIBUTE_INTERNAL;
void LogPrintDrvStatus(uint32_t log_type, uint32_t log_flags, uint32_t channel, struct TDeviceStatus *dev_status) ATTRIBUTE_INTERNAL;
void LogPrintUsbDevices(uint32_t log_type, uint32_t log_flags, struct TCanDevicesList *list, int32_t num_devs) ATTRIBUTE_INTERNAL;

#ifdef __cplusplus
  }
#endif

#endif
