/***************************************************************************
                            usb_hlp.c  -  description
                             -------------------
    begin             : 19.01.2012
    copyright         : (C) 2010 - 2012 by MHS-Elektronik GmbH & Co. KG, Germany
    author            : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************/

/**
    Library to talk to Tiny-CAN devices. You find the latest versions at
       http://www.tiny-can.com/
**/
#include "global.h"
#include <string.h>
#include "util.h"
#include "log.h"
#include "tcan_drv.h"
#include "pnp.h"
#include "errors.h"
#include "index.h"
#include "mhs_thread.h"
#include "usb_hlp.h"

struct TTCanUsbList *TCanUsbList = NULL;
static int32_t LastPnPStatus = -1;
static char *LastUsedSnr = NULL;
struct TTCanUsbList *LastUsedDevice = NULL;


void UsbHlpInit(void)
{
TCanUsbList = NULL;
LastPnPStatus = -1;
LastUsedSnr = NULL;
LastUsedDevice = NULL;
}


void DestroyTCanUsbList(struct TTCanUsbList **list)
{
struct TTCanUsbList *ptr, *l;

if (!list)
  return;
l = *list;
while (l)
  {
  ptr = l->Next;
  safe_free(l->DeviceName);
  safe_free(l->Description);
  safe_free(l->Serial);
  mhs_free(l);
  l = ptr;
  }
*list = NULL;
LastUsedDevice = NULL;
safe_free(LastUsedSnr);  // <*> Past eigentlich hier nicht her
}


struct TTCanUsbList *AddTCanUsbDevice(struct TTCanUsbList **list, char *devicename, char *description,
                          char *serial, uint32_t type, uint32_t idx, const struct TCanModulDesc *modul)
{
struct TTCanUsbList *ptr, *neu;

if (!list)
  return(NULL);
neu = (struct TTCanUsbList *)mhs_calloc(1, sizeof(struct TTCanUsbList));
if (!neu)
  return(NULL);        // Nicht gengend Arbetsspeicher
neu->Next = NULL;
neu->DeviceType = type;
neu->TCanIdx = idx;
neu->DeviceName = mhs_strdup(devicename);
neu->Description = mhs_strdup(description);
neu->Serial = mhs_strdup(serial);
neu->Modul = modul;
neu->Status = PNP_DEVICE_PLUGED;
ptr = *list;
if (!ptr)
  // Liste ist leer
  *list = neu;
else
  { // Neues Element anhngen
  while (ptr->Next != NULL) ptr = ptr->Next;
    ptr->Next = neu;
  }
return(neu);
}


struct TTCanUsbList *GetUsbDevice(char *snr)
{
struct TTCanUsbList *usb_list;

if (!MhsPnP)
  return(NULL);
if (DriverInit == TCAN_DRV_STD_INIT)
  {
  safe_free(LastUsedSnr);
  LastUsedSnr = mhs_strdup(snr);
  }
PnPLock();
// *** Liste der angeschlossenen FTDI-USB Devices erzeugen
UpdateTCanUsbList();
// *** Device Namen u. BaudRate festlegen
usb_list = TCanUsbList;
if ((snr) && (strlen(snr)))
  {  // Device mit entsprechender Snr. suchen
  for (; usb_list; usb_list = usb_list->Next)
    {
    if (usb_list->Status < PNP_DEVICE_PLUGED)
      continue;
    if (!safe_strcmp(snr, usb_list->Serial))
      break;
    }
  }
else
  {  // Nach einen beliebigen Tiny-CAN Modul suchen
  for (; usb_list; usb_list = usb_list->Next)
    {
    if (usb_list->Status < PNP_DEVICE_PLUGED)
      continue;
    if (usb_list->Modul)
      break;
    }
  // Beliebiges FTDI Device auswlen
  if (!usb_list)
    {
    for (usb_list = TCanUsbList; usb_list; usb_list = usb_list->Next)
      {
      if (usb_list->Status < PNP_DEVICE_PLUGED)
        continue;
      }
    }
  }
PnPUnlock();
LastUsedDevice = usb_list;
return(usb_list);
}


int32_t GetDevicePnPStatus(void)
{
struct TTCanUsbList *usb_list;
int32_t plug;

PnPLock();
plug = -1;
//plug_cnt = 0;
if ((LastUsedSnr) && (strlen(LastUsedSnr)))
  {
  for (usb_list = TCanUsbList; usb_list; usb_list = usb_list->Next)
    {
    if (!safe_strcmp(LastUsedSnr, usb_list->Serial))
      {
      if (usb_list->Status < PNP_DEVICE_PLUGED)
        plug = 0;
      else
        plug = 1;
      break;
      }
    }
  }
else if (LastUsedDevice)
  {
  if (LastUsedDevice->Status < PNP_DEVICE_PLUGED)
    {
    plug = 0;
    LastUsedDevice = NULL;
    }
  else
    plug = 1;
  }
else
  {
  plug = 0;
  for (usb_list = TCanUsbList; usb_list; usb_list = usb_list->Next)
    {
    if (usb_list->Status >= PNP_DEVICE_PLUGED)
      {
      plug = 1;
      break;
      }
    }
  }
if ((plug > -1) && (LastPnPStatus != plug))
  LastPnPStatus = plug;
else
  plug = -1;
PnPUnlock();
return(plug);
}
