#ifndef __TAR_DRV_H__
#define __TAR_DRV_H__

#include "global.h"
#include "com_io.h"
#include "sm_drv.h"
#include "tcan_drv.h"

#ifdef __cplusplus
  extern "C" {
#endif

#define CMD_TYPE_EXT1 0x80
#define CMD_TYPE_EXT2 0xC0

#define CMD_GET_STATUS             0x40
#define CMD_GET_STATUS_ALL         0x41

#define CMD_GET_CAN0_TX_STATUS     0x42
#define CMD_GET_CAN1_TX_STATUS     0x43

#define CMD_GET_SPI_I2C_STATUS     0x44

#define CMD_CAN0_SET               0x01 | CMD_TYPE_EXT1
#define CMD_CAN0_GET               0x02 | CMD_TYPE_EXT1
#define CMD_CAN0_SET_MODE          0x03 | CMD_TYPE_EXT1
#define CMD_CAN0_RXD_CLEAR         0x45
#define CMD_CAN0_RXD_GET_COUNT     0x46
#define CMD_CAN0_RXD               0x47
#define CMD_CAN0_TXD_CLEAR         0x48
#define CMD_CAN0_TXD_GET_COUNT     0x49
#define CMD_CAN0_TXD               0x04 | CMD_TYPE_EXT2
#define CMD_CAN0_RXD_SET_FILTER    0x05 | CMD_TYPE_EXT1
#define CMD_CAN0_RXD_IDX_CLEAR     0x06 | CMD_TYPE_EXT1
#define CMD_CAN0_RXD_IDX_GET_COUNT 0x07 | CMD_TYPE_EXT1
#define CMD_CAN0_RXD_IDX           0x08 | CMD_TYPE_EXT1
#define CMD_CAN0_TXD_IDX_SET_TIMER 0x09 | CMD_TYPE_EXT1
#define CMD_CAN0_TXD_IDX_CLEAR     0x0A | CMD_TYPE_EXT1
#define CMD_CAN0_TXD_IDX_GET_COUNT 0x0B | CMD_TYPE_EXT1
#define CMD_CAN0_TXD_IDX           0x0C | CMD_TYPE_EXT1

#define CMD_CAN1_SET               0x0D | CMD_TYPE_EXT1
#define CMD_CAN1_GET               0x0E | CMD_TYPE_EXT1
#define CMD_CAN1_SET_MODE          0x0F | CMD_TYPE_EXT1
#define CMD_CAN1_RXD_CLEAR         0x4A
#define CMD_CAN1_RXD_GET_COUNT     0x4B
#define CMD_CAN1_RXD               0x4C
#define CMD_CAN1_TXD_CLEAR         0x4D
#define CMD_CAN1_TXD_GET_COUNT     0x4E
#define CMD_CAN1_TXD               0x10 | CMD_TYPE_EXT2
#define CMD_CAN1_RXD_SET_FILTER    0x11 | CMD_TYPE_EXT1
#define CMD_CAN1_RXD_IDX_CLEAR     0x12 | CMD_TYPE_EXT1
#define CMD_CAN1_RXD_IDX_GET_COUNT 0x13 | CMD_TYPE_EXT1
#define CMD_CAN1_RXD_IDX           0x14 | CMD_TYPE_EXT1
#define CMD_CAN1_TXD_IDX_SET_TIMER 0x15 | CMD_TYPE_EXT1
#define CMD_CAN1_TXD_IDX_CLEAR     0x16 | CMD_TYPE_EXT1
#define CMD_CAN1_TXD_IDX_GET_COUNT 0x17 | CMD_TYPE_EXT1
#define CMD_CAN1_TXD_IDX           0x18 | CMD_TYPE_EXT1

#define CMD_CAN0_RXD_ALL           0x4F
#define CMD_CAN1_RXD_ALL           0x50

#define CMD_SPI_SET                0x19 | CMD_TYPE_EXT1
#define CMD_SPI_GET                0x1A | CMD_TYPE_EXT1
#define CMD_SPI_SET_MODE           0x1B | CMD_TYPE_EXT1
#define CMD_SPI_TXD                0x1C | CMD_TYPE_EXT2

#define CMD_I2C_SET                0x1D | CMD_TYPE_EXT1
#define CMD_I2C_GET                0x1E | CMD_TYPE_EXT1
#define CMD_I2C_SET_MODE           0x1F | CMD_TYPE_EXT1
#define CMD_I2C_TXD                0x20 | CMD_TYPE_EXT2

#define CMD_GET_INPUT              0x21 | CMD_TYPE_EXT1
#define CMD_SET_OUTPUT             0x22 | CMD_TYPE_EXT1
#define CMD_GET_AD                 0x51
#define CMD_GET_INFO               0x23 | CMD_TYPE_EXT1

#define CMD_PING                   0x7E

#define CMD_START_READ_INFO        0x52
#define CMD_READ_INFO              0x53

#define CMD_COM                    0x24 | CMD_TYPE_EXT1
#define CMD_SET_TIMESTAMP          0x25 | CMD_TYPE_EXT1


/**********************************/
/* Index für CANx_SET & CANx_GET  */
/**********************************/
#define CAN_SPEED_INDEX         0
#define CAN_SPEED_BTR           1
#define CAN_TX_ACK_ENABLE       2
#define CAN_MODE_INDEX          3
#define CAN_TXD_FIFO_LIMIT      4
#define CAN_FILTER0_INDEX       5
#define CAN_FILTER1_INDEX       6
#define CAN_FILTER2_INDEX       7
#define CAN_FILTER3_INDEX       8
#define CAN_FILTER4_INDEX       9
#define CAN_FILTER5_INDEX       10
#define CAN_FILTER6_INDEX       11
#define CAN_FILTER7_INDEX       12
#define CAN_TXD_BOX0_INDEX      13
#define CAN_TXD_BOX1_INDEX      14
#define CAN_TXD_BOX2_INDEX      15
#define CAN_TXD_BOX3_INDEX      16
#define CAN_TXD_BOX4_INDEX      17
#define CAN_TXD_BOX5_INDEX      18
#define CAN_TXD_BOX6_INDEX      19
#define CAN_TXD_BOX7_INDEX      20
#define CAN_OPT_INDEX           21
#define CAN_ERROR_LOG_ENABLE    22

#define TAR_CAN_STATUS_OK      0   // Controller Ok
#define TAR_CAN_STATUS_WARNING 1   // Error warning
#define TAR_CAN_STATUS_PASSIV  2   // Error passiv
#define TAR_CAN_STATUS_BUS_OFF 3   // Bus Off


/***************************************/
/*               Status                */
/***************************************/
#define TAR_CAN_RXD_PEND   0x08
#define TAR_CAN_RXD_OV     0x10

#define TAR_CAN_TXD_MASK   0x60
#define TAR_CAN_TXD_EMPTY  0x00
#define TAR_CAN_TXD_PEND   0x20
#define TAR_CAN_TXD_LIMIT  0x40
#define TAR_CAN_TXD_OV     0x60

//#define TAR_CAN_TXD_LIMIT  0x40
#define TAR_CAN_TXD_ERROR  0x80

#define TXD_STATUS_EMPTY 0
#define TXD_STATUS_PEND  1
#define TXD_STATUS_LIMIT 2
#define TXD_STATUS_OV    3

struct TTarStatus0Flags
  {
  unsigned CanBusStatusBits:3;
  unsigned CanRxDPend:1;
  unsigned CanRxDOv:1;
  unsigned CanTxDStatus:2;
  unsigned CanTxDError:1;
  };

union TTarStatus0
  {
  unsigned char Byte;
  struct TTarStatus0Flags Flag;
  };

struct TTarStatus1Flags
  {
  unsigned BusFailure:1;  
  };

union TTarStatus1
  {
  unsigned char Byte;
  struct TTarStatus1Flags Flag;
  };

struct TTarStatus
  {
  union TTarStatus0 Status;
  union TTarStatus1 ExStatus;
  };
  

/***************************************/
/*            CAN Status               */
/***************************************/
struct TTarInfoVar
  {
  unsigned char Size;
  unsigned char Type;
  uint16_t Index;
  char Data[128];
  };


int32_t TARGetStatus(TCanDevice *dev, struct TTarStatus *tar_status) ATTRIBUTE_INTERNAL;
int32_t TARSetFilter(TCanDevice *dev, uint32_t index, struct TMsgFilter *msg_filter) ATTRIBUTE_INTERNAL;
int32_t TARCommSync(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t TARPing(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t TARInfoStartRead(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t TARInfoGetNext(TCanDevice *dev, struct TTarInfoVar *info) ATTRIBUTE_INTERNAL;
int32_t TARSetCANMode(TCanDevice *dev, unsigned char can_mode) ATTRIBUTE_INTERNAL;
int32_t TARCanSetTxAckEnable(TCanDevice *dev, unsigned char enable) ATTRIBUTE_INTERNAL;
int32_t TARCanSetErrorLogEnable(TCanDevice *dev, unsigned char enable) ATTRIBUTE_INTERNAL;
int32_t TARCanSetSpeed(TCanDevice *dev, uint32_t speed) ATTRIBUTE_INTERNAL;
int32_t TARCanSetBtr(TCanDevice *dev, uint32_t btr) ATTRIBUTE_INTERNAL;
int32_t TARCanSetIntervall(TCanDevice *dev, uint32_t index, uint32_t intervall) ATTRIBUTE_INTERNAL;
int32_t TARCANReset(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t TARRxFifoClear(TCanDevice *dev, uint32_t index) ATTRIBUTE_INTERNAL;
int32_t TARTxFifoPut(TCanDevice *dev, uint32_t index, struct TCanMsg *msgs, uint16_t count) ATTRIBUTE_INTERNAL;
int32_t TARTxFifoClear(TCanDevice *dev, uint32_t index) ATTRIBUTE_INTERNAL;
int32_t TARSetTimeStamp(TCanDevice *dev, unsigned char enabled) ATTRIBUTE_INTERNAL;

#ifdef __cplusplus
  }
#endif

#endif
