/***************************************************************************
                          tar_drv.c  -  description
                             -------------------
    begin             : 01.11.2010
    last modify       : 01.01.2016    
    copyright         : (C) 2010 - 2016 by MHS-Elektronik GmbH & Co. KG, Germany
    author            : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************/

/**
    Library to talk to Tiny-CAN devices. You find the latest versions at
       http://www.tiny-can.com/
**/
#include "global.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "util.h"
#include "info.h"
#include "errors.h"
#include "log.h"
#include "can_core.h"
#include "can_fifo.h"
#include "can_puffer.h"
#include "mhs_thread.h"
#include "mhs_cmd.h"
#include "mhs_obj.h"
#include "var_obj.h"
#include "usb_hlp.h"
#include "index.h"
#include "pnp.h"
#include "api_cp.h"
#include "tar_drv.h"
#include "tcan_com.h"
#include "tcan_drv.h"


const struct TCanModulDesc CanModulTable[] =
  //   ID    | USB-Description |BaudRate|USB-Device        |St. |HW-T |Tx |Err |Features|Fifo|Fifo |Filter|Puffer|CAN-Clock
  //         |                 |        |  Type            |Size|Stamp|ACK|Msgs| Flags  |Size|Limit|Count |Count   (MHz)
{{0x43414E00, "Tiny-CAN I-XL",  3000000, FT_DEVICE_X_SERIES,  1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN I-XL
 {0x43414E01, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      24    },  // Tiny-CAN I
 {0x43414E81, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      16    },  // Tiny-CAN I / 10kBit
 {0x43414E41, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      24    },  // Tiny-CAN IB
 {0x43414EC1, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      16    },  // Tiny-CAN IB / 10kBit
 {0x43414E02, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      24    },  // Tiny-CAN II
 {0x43414E82, NULL,             921600,  FT_DEVICE_232R,      1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      16    },  // Tiny-CAN II / 10 kBit
 {0x43414E42, "Tiny-CAN II-XL", 3000000, FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN II-XL
 {0x43414E03, NULL,             921600,  FT_DEVICE_BM,        1,   1,   0,  0,   0x8000,  72,  36,    8,    16,      24    },  // Tiny-CAN III
 {0x43414E83, NULL,             921600,  FT_DEVICE_BM,        1,   1,   0,  0,   0x8000,  72,  36,    8,    16,      16    },  // Tiny-CAN III / 10 kBit
 {0x43414E43, "Tiny-CAN III-XL",3000000, FT_DEVICE_BM,        1,   1,   1,  0,   0x8007,  72,  36,    8,    16,      16    },  // Tiny-CAN III-XL
 {0x43414E04, "Tiny-CAN M1",    3000000, FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN M1
 {0x43414E05, NULL,             921600,  FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  36,  18,    8,     4,      16    },  // Tiny-CAN M232
 {0x43414E06, "Tiny-CAN IV-XL", 8000000, FT_DEVICE_2232H,     1,   1,   1,  1,   0x800F,  72,  36,    4,     8,      16    },  // Tiny-CAN IV-XL
 {0x43414E07, "Tiny-CAN LS",    3000000, FT_DEVICE_X_SERIES,  2,   1,   1,  1,   0x800F,  72,  36,    4,     4,      16    },  // Tiny-CAN LS
 {0x43414E08, "Tiny-CAN M2",    3000000, FT_DEVICE_X_SERIES,  1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN M2
 {0x00000000, NULL,             0,        0,                  0,   0,   0,  0,   0x0000,  0,    0,    0,     0,      0     }}; // Ende Tabelle

static const struct TCanModulDesc DefaultModul =
 {0x43414E01, NULL,             921600,   FT_DEVICE_232R,     1,   0,   0,  0,   0x0000,  36,  18,    4,     4,      24    };  // Tiny-CAN I


const struct TCanModulDesc ExCanModulTable[] =
  //   ID    | USB-Description |BaudRate|USB-Device        |St. |HW-T |Tx |Err |Features|Fifo|Fifo |Filter|Puffer|CAN-Clock
  //         |                 |        |  Type            |Size|Stamp|ACK|Msgs| Flags  |Size|Limit|Count |Count   (MHz)
{{0x43414E00, "Tiny-CAN I-XL",  3000000, FT_DEVICE_X_SERIES,  1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN I-XL
 {0x43414E42, "Tiny-CAN II-XL", 3000000, FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN II-XL
 {0x43414E43, "Tiny-CAN III-XL",3000000, FT_DEVICE_BM,        1,   1,   1,  0,   0x8007,  72,  36,    8,    16,      16    },  // Tiny-CAN III-XL
 {0x43414E04, "Tiny-CAN M1",    3000000, FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN M1
 {0x43414E05, NULL,             921600,  FT_DEVICE_232R,      1,   0,   1,  0,   0x0005,  36,  18,    8,     4,      16    },  // Tiny-CAN M232
 {0x43414E06, "Tiny-CAN IV-XL", 8000000, FT_DEVICE_2232H,     1,   1,   1,  1,   0x800F,  72,  36,    4,     8,      16    },  // Tiny-CAN IV-XL
 {0x43414E07, "Tiny-CAN LS",    3000000, FT_DEVICE_X_SERIES,  2,   1,   1,  1,   0x800F,  72,  36,    4,     4,      16    },  // Tiny-CAN LS
 {0x43414E08, "Tiny-CAN M2",    3000000, FT_DEVICE_X_SERIES,  1,   0,   1,  0,   0x0005,  72,  36,    4,     4,      16    },  // Tiny-CAN M2
 {0x00000000, NULL,             0,        0,                  0,   0,   0,  0,   0x0000,  0,    0,    0,     0,      0     }}; // Ende Tabelle

/*
Initialisierungsvariablen

Bezeichner            | Beschreibung                                    | Initialisierung
======================+=================================================+==========================
MainThreadPriority    | 0 = THREAD_PRIORITY_NORMAL                      | MAIN_THREAD_PRIORITY_INIT
                      | 1 = THREAD_PRIORITY_ABOVE_NORMAL                |
                      | 2 = THREAD_PRIORITY_HIGHEST                     |
                      | 3 = THREAD_PRIORITY_TIME_CRITICAL               |
                      | 4 = THREAD_PRIORITY_REALTIME                    |
----------------------+-------------------------------------------------+--------------------------
TimeStampMode         | 0 = Disabled                                    | TIME_STAMP_MODE_INIT
                      | 1 = Software Time Stamps                        |
                      | 2 = Hardware Time Stamps, UNIX-Format           |
                      | 3 = Hardware Time Stamps                        |
                      | 4 = Hardware Time-Stamps verwenden wenn         |
                      |     verfgbar, ansonsten Software Time-Stamps   |
                      | Achtung: Ist der Bus gestartet (RUN) wird die <*> gendert ! 
                      | Einstellung erst nach erneuten start wirksam    |  
----------------------+-------------------------------------------------+--------------------------
CanRxDFifoSize        | Gre des Empfangsfifos in Messages             | CAN_RXD_FIFO_SIZE
----------------------+-------------------------------------------------+--------------------------
CanTxDFifoSize        | Gre des Sendefifos in Messages                | CAN_TXD_FIFO_SIZE
======================+=================================================+==========================
*/

static const uint32_t MAIN_THREAD_PRIORITY_INIT = {4};  // <*> einstellung von 3 auf 4 gendert
static const unsigned char TIME_STAMP_MODE_INIT = {0};
static const uint32_t CAN_TXD_FIFO_SIZE_INIT    = {0};
static const uint32_t CAN_RXD_FIFO_SIZE_INIT    = {0};

/*
Port Open Variablen

Bezeichner            | Beschreibung                                    | Initialisierung
======================+=================================================+==========================
ComDrvType            | 0 = RS232 Schnittstelle                         | COM_DRV_TYPE_INIT
                      | 1 = USB (FTDI)                                  |
----------------------+-------------------------------------------------+--------------------------
Port                  | Serielle Schnittstelle 1 = COM1 - 4 = COM 4     | PORT_INIT
----------------------+-------------------------------------------------+--------------------------
ComDeviceName         | Device Name (Linux z.b. /dev/ttyS0              | COM_DEVICE_NAME_INIT
----------------------+-------------------------------------------------+--------------------------
BaudRate              | Baudrate, z.B. 34800 = 34800 Baud               | PORT_BAUD_INIT
----------------------+-------------------------------------------------+--------------------------
VendorId              | USB-Produkt Id                                  | VENDOR_ID_INIT
----------------------+-------------------------------------------------+--------------------------
ProductId             | USB-Vendor Id                                   | PRODUCT_ID_INIT
----------------------+-------------------------------------------------+--------------------------
Snr                   | Seriennummer CAN-Device                         | SNR_INIT
======================+=================================================+==========================
*/

static const int32_t COM_DRV_TYPE_INIT = {1};
static const uint16_t PORT_INIT = {1};             // COM 1
static const char COM_DEVICE_NAME_INIT[] = {""};
static const uint32_t PORT_BAUD_INIT = {0};
static const int32_t VENDOR_ID_INIT = {0};         // {0x0403};
static const int32_t PRODUCT_ID_INIT = {0};        // {0x6001};
static const char SNR_INIT[] = {""};

/*
Option Variablen

======================+=================================================+==========================
CanTxAckEnable        | CAN TxD ACK Enablen                             | CAN_TX_ACK_ENABLE_INIT
----------------------+-------------------------------------------------+--------------------------
CanErrorMsgsEnable    | CAN Fehler Nachrichten in FIFO schreiben        | CAN_ERR_MSGS_ENABLE_INIT
----------------------+-------------------------------------------------+--------------------------
CanSpeed1             | CAN bertragungsgeschwindigkeit in kBit/s       | CAN_SPEED_INIT
                      | z.B. 100 = 100kBit/s, 1000 = 1MBit/s            |
----------------------+-------------------------------------------------+--------------------------
CanSpeed1User         |
----------------------+-------------------------------------------------+--------------------------
ExecuteCommandTimeout | Wartezeit fr Kommando ausfhrung in ms         | EXECUTE_COMMAND_TIMEOUT_INIT
----------------------+-------------------------------------------------+--------------------------
LowPollIntervall      | Hardware Polling Intervall in ms                | LOW_POLL_INTERVALL_INIT
----------------------+-------------------------------------------------+--------------------------
IdlePollIntervall     | Hardware Polling Intervall in ms                | IDLE_POLL_INTERVALL_INIT  // <*> NEU!!! 
----------------------+-------------------------------------------------+--------------------------
HighPollIntervall     | Hardware Polling Intervall in ms CAN Bus run    | HIGH_POLL_INTERVALL_INIT
----------------------+-------------------------------------------------+--------------------------
FilterReadIntervall   | Filter Messages alle x ms einlesen              | FILTER_READ_INTERVALL_INIT
======================+=================================================+==========================
*/

static const uint16_t CAN_SPEED_INIT     = {125};               // 125 kBit/s
static const uint32_t CAN_SPEED_USER_INIT = {0};
static const uint32_t EXECUTE_COMMAND_TIMEOUT_INIT = {6000};    // 6 Sekunden max. warten
static const uint32_t LOW_POLL_INTERVALL_INIT      = {250};
static const uint32_t IDLE_POLL_INTERVALL_INIT     = {0xFFFFFFFF}; // Globale Einstellungen verwenden 
static const uint32_t HIGH_POLL_INTERVALL_INIT     = {0xFFFFFFFF}; //          - " - 
static const uint32_t FILTER_READ_INTERVALL_INIT   = {0xFFFFFFFF}; //          - " -
static const unsigned char CAN_TX_ACK_ENABLE_INIT = {0};
static const unsigned char CAN_ERR_MSGS_ENABLE_INIT = {0};
static const unsigned char AUTO_STOP_CAN_INIT = {0};


#define RS232_DEF_BAUDRATE 921600
#define FTDI_DEF_BAUDRATE 921600

static const struct TValueDescription DeviceValues[] = {
// Name                    | Type     |MaxSize| Access             | Default
  // *** Initialisierungsvariablen
  {"MainThreadPriority",    VT_ULONG,     0,   M_RD_ALL | M_WR_INIT, &MAIN_THREAD_PRIORITY_INIT},
  {"TimeStampMode",         VT_UBYTE,     0,   M_RD_ALL | M_WR_INIT, &TIME_STAMP_MODE_INIT},
  {"CanRxDFifoSize",        VT_ULONG,     0,   M_RD_ALL | M_WR_INIT, &CAN_TXD_FIFO_SIZE_INIT},
  {"CanTxDFifoSize",        VT_ULONG,     0,   M_RD_ALL | M_WR_INIT, &CAN_RXD_FIFO_SIZE_INIT},
  // *** Port Open Variablen
  {"ComDrvType",            VT_LONG,      0,   M_RD_ALL | M_WR_INIT, &COM_DRV_TYPE_INIT},
  {"Port",                  VT_LONG,      0,   M_RD_ALL | M_WR_INIT, &PORT_INIT},
  {"DeviceName",            VT_STRING,    0,   M_RD_ALL | M_WR_INIT, &COM_DEVICE_NAME_INIT},
  {"BaudRate",              VT_ULONG,     0,   M_RD_ALL | M_WR_INIT, &PORT_BAUD_INIT},
  {"VendorId",              VT_LONG,      0,   M_RD_ALL | M_WR_INIT, &VENDOR_ID_INIT},
  {"ProductId",             VT_LONG,      0,   M_RD_ALL | M_WR_INIT, &PRODUCT_ID_INIT},
  {"Snr",                   VT_STRING,    0,   M_RD_ALL | M_WR_INIT, &SNR_INIT},
  // *** Option Variablen
  {"CanSpeed1",             VT_UWORD,     0,   M_RD_ALL | M_WR_ALL,  &CAN_SPEED_INIT},
  {"CanSpeed1User",         VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &CAN_SPEED_USER_INIT},
  {"ExecuteCommandTimeout", VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &EXECUTE_COMMAND_TIMEOUT_INIT},
  {"LowPollIntervall",      VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &LOW_POLL_INTERVALL_INIT},
  {"IdlePollIntervall",     VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &IDLE_POLL_INTERVALL_INIT}, // <*> NEU!!!
  {"HighPollIntervall",     VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &HIGH_POLL_INTERVALL_INIT},
  {"FilterReadIntervall",   VT_ULONG,     0,   M_RD_ALL | M_WR_ALL,  &FILTER_READ_INTERVALL_INIT},
  {"CanTxAckEnable",        VT_UBYTE,     0,   M_RD_ALL | M_WR_ALL,  &CAN_TX_ACK_ENABLE_INIT},
  {"CanErrorMsgsEnable",    VT_UBYTE,     0,   M_RD_ALL | M_WR_ALL,  &CAN_ERR_MSGS_ENABLE_INIT},
  {"AutoStopCan",           VT_UBYTE,     0,   M_RD_ALL | M_WR_ALL,  &AUTO_STOP_CAN_INIT},     // <*> NEU!!!
  {NULL,                    0,            0,   0,                    NULL}};


static void MainUpdateStatusProc(TCanDevice *dev);

static void MainThreadExecute(TMhsThread *thread);

static void SetCanSpeed1CB(TObjValue *obj, void *user_data);
static void SetCanSpeed1UserCB(TObjValue *obj, void *user_data);
static void SetCanTxAckEnableCB(TObjValue *obj, void *user_data);
static void SetCanSetErrorLogEnableCB(TObjValue *obj, void *user_data);
static void SetFilterReadIntervallCB(TObjValue *obj, void *user_data);
static void SetHighPollIntervallCB(TObjValue *obj, void *user_data);
static void SetIdlePollIntervallCB(TObjValue *obj, void *user_data);
static void SetIntervallBoxCB(TMhsObj *obj, void *user_data);
static void SetFilterBoxCB(TMhsObj *obj, void *user_data);


static void FreeModulDesc(struct TCanModulDesc **desc)
{
struct TCanModulDesc *desc_;

if (!desc)
  return;
if (!(desc_ = *desc))
  return;
safe_free(desc_->Description);
mhs_free(desc_);
*desc = NULL;
}


static struct TCanModulDesc *CopyModulDesc(const struct TCanModulDesc *src_desc)
{
struct TCanModulDesc *desc;

if (!(desc = (struct TCanModulDesc *)mhs_calloc(1, sizeof(struct TCanModulDesc))))
  return(NULL);
memcpy(desc, src_desc, sizeof(struct TCanModulDesc));
desc->Description = mhs_strdup(src_desc->Description);
return(desc);
}


/*******************************************************************************/
/*                           CAN Device erzeugen                               */
/*******************************************************************************/
TCanDevice *CreateCanDevice(void)
{
int32_t err;
TCanDevice *dev;
TMhsObjContext *context;
TMhsObj *obj;
#if (TX_PUFFER_MAX > 0) || (RX_FILTER_MAX > 0)
uint32_t i;
TObjCanPuffer *puffer;
#endif
#if RX_FILTER_MAX > 0
TMhsThread *thread;
#endif

err = 0;
if (!(dev = device_create()))
  return(NULL);
dev->MainThread = mhs_create_thread(MainThreadExecute, dev, 0, 0);
mhs_event_set_event_mask((TMhsEvent *)dev->MainThread, MHS_ALL_EVENTS);
#if TX_PUFFER_MAX > 0
// **** Intervall Boxen anlegen
context = mhs_obj_context_new("", (TMhsEvent *)dev->MainThread);
dev->TxPufferContext = context;
for (i = 1; i <= TX_PUFFER_MAX; i++)
  {
  if (!(puffer = mhs_can_puffer_create(OBJ_CAN_TX_PUFFER, i | INDEX_TXT_FLAG, NULL, context)))
    {
    err = -1;
    break;
    }
  mhs_can_puffer_set_intervall(puffer, 0);
  mhs_object_cmd_event_connect((TMhsObj *)puffer, 0, (TMhsObjectCB)&SetIntervallBoxCB, (void *)dev);
  mhs_object_event_connect((TMhsObj *)puffer, 0, (TMhsObjectCB)&SetIntervallBoxCB, (void *)dev);
  }
#endif
#if RX_FILTER_MAX > 0
context = mhs_obj_context_new("", (TMhsEvent *)dev->MainThread);
dev->RxFilterContext = context;
for (i = 1; i <= RX_FILTER_MAX; i++)
  {
  if (!(puffer = mhs_can_puffer_create(OBJ_CAN_RX_PUFFER, i, NULL, context)))
    {
    err = -1;
    break;
    }
  if ((thread = GetApiCpTheread()))
    (void)mhs_object_events_set((TMhsObj *)puffer, (TMhsEvent *)thread, EVENT_RX_FILTER_MESSAGES);
  mhs_object_cmd_event_connect((TMhsObj *)puffer, 0, (TMhsObjectCB)&SetFilterBoxCB, (void *)dev);
  }
#endif
if (!err)
  {
  dev->IoContext = mhs_obj_context_new("", NULL);
  context = mhs_obj_context_new("", (TMhsEvent *)dev->MainThread);
  dev->Context = context;
  err = mhs_values_create_from_list(context, DeviceValues);
  mhs_cmd_create(&dev->Cmd);
  }
if (err >= 0)
  {
  obj = mhs_object_get_by_name("CanSpeed1", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetCanSpeed1CB, (void *)dev);
  obj = mhs_object_get_by_name("CanSpeed1User", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetCanSpeed1UserCB, (void *)dev);
  obj = mhs_object_get_by_name("CanTxAckEnable", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetCanTxAckEnableCB, (void *)dev);
  obj = mhs_object_get_by_name("CanErrorMsgsEnable", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetCanSetErrorLogEnableCB, (void *)dev);
  obj = mhs_object_get_by_name("FilterReadIntervall", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetFilterReadIntervallCB, (void *)dev);
  (void)mhs_object_set_cmd_event(obj);
  obj = mhs_object_get_by_name("HighPollIntervall", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetHighPollIntervallCB, (void *)dev);
  (void)mhs_object_set_cmd_event(obj);
  obj = mhs_object_get_by_name("IdlePollIntervall", context);
  mhs_object_cmd_event_connect(obj, 0, (TMhsObjectCB)&SetIdlePollIntervallCB, (void *)dev);
  (void)mhs_object_set_cmd_event(obj);
  }
if (err < 0)
  (void)device_destroy(dev->Index, 1);
else
  dev->DeviceStatus.DrvStatus = DRV_STATUS_INIT;
return(dev);
}


/*******************************************************************************/
/*                           CAN Device lschen                                */
/*******************************************************************************/
void DestroyCanDevice(TCanDevice **device)
{
TCanDevice *dev;

if (!device)
  return;
dev = *device;
if (dev)
  {
  (void)mhs_cmd_execute(dev, MCMD_DEVICE_CLOSE, NULL, 0, 1000);   // <*>
  (void)mhs_join_thread(dev->MainThread, 5000);  // Timeout 5 Sek.
  clear_device_from_list(dev->Index);
  dev->DrvStatusEvent = NULL;
  CloseCanDevice(dev);
  safe_free(dev->DrvInfoHwStr);
  InfoVarDestroy(&dev->InfoVars);
  mhs_obj_context_destroy(&dev->Context);
  mhs_obj_context_destroy(&dev->IoContext);
#if TX_PUFFER_MAX > 0
  mhs_obj_context_destroy(&dev->TxPufferContext);
#endif
#if RX_FILTER_MAX
  mhs_obj_context_destroy(&dev->RxFilterContext);
#endif
  mhs_cmd_destroy(&dev->Cmd);
  //clear_device_from_list(dev->Index);
  (void)mhs_destroy_thread(&dev->MainThread, 1);
  mhs_free(dev);
  }
*device = NULL;
}


int32_t GetDeviceInfo(TCanDevice *dev, struct TCanDeviceInfo *device_info)
{
uint32_t snr;
char *desc;

if (!device_info)
  return(ERR_PARAM);
desc = GetInfoValueAsString(GetInfoByIndex(dev, 0x0001)); // Hardware ID String
if (desc)
  safe_strcpy(device_info->Description, DESCRIPTION_MAX_SIZE, desc);
device_info->FirmwareVersion = GetInfoValueAsULong(GetInfoByIndex(dev, 0x1002)); // Version
snr = GetInfoValueAsULong(GetInfoByIndex(dev, 0x0000)); // Hardware Snr
safe_sprintf(device_info->SerialNumber, SERIAL_NUMBER_MAX_SIZE, "%08X", snr);

if (dev->ModulDesc)
  {
  device_info->ModulFeatures.CanClock = dev->ModulDesc->CanClock;
  device_info->ModulFeatures.Flags = dev->ModulDesc->CanFeaturesFlags;
  device_info->ModulFeatures.CanChannelsCount = 1;
  device_info->ModulFeatures.HwRxFilterCount = dev->ModulDesc->HwRxFilterCount;
  device_info->ModulFeatures.HwTxPufferCount = dev->ModulDesc->HwTxPufferCount;
  device_info->HwId = dev->ModulDesc->HwId;
  }
else
  {
  device_info->ModulFeatures.CanClock = 24;
  device_info->ModulFeatures.Flags = 0;
  device_info->ModulFeatures.CanChannelsCount = 1;
  device_info->ModulFeatures.HwRxFilterCount = 4;
  device_info->ModulFeatures.HwTxPufferCount = 4;
  device_info->HwId = 0;
  }
return(0);
}


static int32_t SetupCanDevice(TCanDevice *dev)
{
int32_t res;
struct TTCanUsbList *usb_device;
TMhsObjContext *context, *io_context;
int32_t usr_drv_type, usr_port, usr_vendor_id, usr_product_id;
uint32_t baud_rate, usr_baud_rate;
char *usr_snr;
#ifndef __WIN32__
char *usr_device_name;
#endif

io_context = dev->IoContext;
context = dev->Context;
usr_drv_type = mhs_value_get_as_long("ComDrvType", ACCESS_INIT, context);
usr_port = mhs_value_get_as_long("Port", ACCESS_INIT, context);
usr_baud_rate = mhs_value_get_as_ulong("BaudRate", ACCESS_INIT, context);
usr_snr = mhs_value_get_as_string("Snr", ACCESS_INIT, context);
usr_vendor_id = mhs_value_get_as_long("VendorId", ACCESS_INIT, context);
usr_product_id = mhs_value_get_as_long("ProductId", ACCESS_INIT, context);
#ifndef __WIN32__
usr_device_name = mhs_value_get_as_string("DeviceName", ACCESS_INIT, context);
#endif

FreeModulDesc(&dev->ModulDesc);
baud_rate = 0;
usb_device = NULL;
res = 0;
if (!usr_drv_type)
  {  // RS 232
  if (usr_baud_rate)
    baud_rate = usr_baud_rate;
  else
    baud_rate = RS232_DEF_BAUDRATE;
  (void)mhs_value_set_as_long("PortNr", usr_port, ACCESS_INIT, 0, io_context);
  }
else
  {  // USB FTDI
  // Plug & Play aus ?
  if ((usr_vendor_id) || (usr_product_id))
    {  // ja
    if (usr_vendor_id)
      (void)mhs_value_set_as_long("VendorId", usr_vendor_id, ACCESS_INIT, 0, io_context);
    if (usr_product_id)
      (void)mhs_value_set_as_long("ProductId", usr_product_id, ACCESS_INIT, 0, io_context);
    }
#ifdef __WIN32__
  else
#else
  else if (!((usr_device_name) && (strlen(usr_device_name))))
#endif
    {
    if ((usb_device = GetUsbDevice(usr_snr)))
      {
      res = 1;
      if (usb_device->Modul)
        dev->ModulDesc = CopyModulDesc(usb_device->Modul);
      else
        dev->ModulDesc = CopyModulDesc(&DefaultModul);
      if (dev->ModulDesc->Description)
        (void)mhs_value_set_as_string("UsbDescription", dev->ModulDesc->Description, ACCESS_INIT, 0, io_context);
#ifndef __WIN32__
      usr_device_name = usb_device->DeviceName;
#endif
      usr_snr = usb_device->Serial;
      baud_rate = dev->ModulDesc->BaudRate;
      }
    }
  // **** Device Name u. Baudrate durch "User Werte" berschreiben
#ifndef __WIN32__
// ****** Linux, Mac
  if ((usr_device_name) && (strlen(usr_device_name)))
    (void)mhs_value_set_as_string("DeviceName", usr_device_name, ACCESS_INIT, 0, io_context);
#endif
  if (usr_baud_rate)
    baud_rate = usr_baud_rate;
  if (!baud_rate)
    baud_rate = FTDI_DEF_BAUDRATE;
  (void)mhs_value_set_as_string("SerialNr", usr_snr, ACCESS_INIT, 0, io_context);
  if (usb_device)
    (void)mhs_value_set_as_ulong("UsbDeviceType", usb_device->DeviceType, ACCESS_INIT, 0, io_context);
  else
    (void)mhs_value_set_as_ulong("UsbDeviceType", FT_DEVICE_232R, ACCESS_INIT, 0, io_context);
  }
(void)mhs_value_set_as_ulong("BaudRate", baud_rate, ACCESS_INIT, 0, io_context);
return(res);
}


/***************************************************************/
/* OpenCanDevice                                               */
/***************************************************************/
int32_t OpenCanDevice(TCanDevice *dev)
{
#ifdef ENABLE_LOG_SUPPORT
TMhsObjContext *context, *io_context, *main_context;
uint32_t baud_rate;
#ifdef __WIN32__
int32_t port;
#endif
char *usb_description;
struct TTarStatus tar_status;

#ifndef __WIN32__
// ****** Linux, Mac
char *device_name;
#endif
#endif
char *snr;
int32_t res, drv_type, i;
uint32_t priority, hw_id, modul_id;
const struct TCanModulDesc *modul_desc;
struct TInfoVarList *info;
uint32_t value;

CloseCanDevice(dev);
context = dev->Context;
io_context = dev->IoContext;
main_context = can_main_get_context();
if (mhs_value_get_status("MainThreadPriority", context) & MHS_VAL_FLAG_WRITE)
  priority = mhs_value_get_as_ulong("MainThreadPriority", ACCESS_INIT, context);
else  
  priority = mhs_value_get_as_ulong("MainThreadPriority", ACCESS_INIT, main_context);
drv_type = mhs_value_get_as_long("ComDrvType", ACCESS_INIT, context);
// Device ffnen
dev->Io = ComIoCreate(drv_type, io_context);
i = SetupCanDevice(dev);
snr = mhs_value_get_as_string("SerialNr", ACCESS_INIT, io_context);
#ifdef ENABLE_LOG_SUPPORT
baud_rate = mhs_value_get_as_ulong("BaudRate", ACCESS_INIT, io_context);
#ifdef __WIN32__
port = mhs_value_get_as_long("PortNr", ACCESS_INIT, io_context);
#endif
usb_description = mhs_value_get_as_string("UsbDescription", ACCESS_INIT, io_context);
#ifndef __WIN32__
// ****** Linux, Mac
device_name = mhs_value_get_as_string("DeviceName", ACCESS_INIT, io_context);
#endif
#endif
if (i)
  i = 4;    // ffnen des Devices 3 mal versuchen <*> von 7 auf 4 gendert
else
  i = 1;    // Nur 1 mal versuchen
for (; i; i--)
  {
#ifdef __WIN32__
  if ((res = ComOpen(dev->Io, NULL, 0, 0)) >= 0)
#else
  if ((res = ComOpen(dev->Io)) >= 0)
#endif
    break;
  if (i > 1)
    mhs_sleep(1000);  // 1000 ms Warten
  }
if (res < 0)
  {  // Fehler
#ifdef ENABLE_LOG_SUPPORT
  if (!drv_type)   // RS232
    {
#ifdef __WIN32__
// ****** Windows
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (Port:%d Baudrate:%d): [%d] %s", port,
      baud_rate, dev->Io->ErrorCode, dev->Io->ErrorString);
#else
// ****** Linux, Mac
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (Device:%s Baudrate:%d): [%d] %s", device_name,
      baud_rate, dev->Io->ErrorCode, dev->Io->ErrorString);
#endif
    }
  else
    {
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (%s Snr.:%s): [%d] %s",
      usb_description, snr, dev->Io->ErrorCode, dev->Io->ErrorString);
    }
#endif
  dev->DeviceStatus.DrvStatus = DRV_STATUS_PORT_NOT_OPEN; // Port kann nicht geffnet werden
  return(ERR_HARDWARE_OFFLINE);
  }
  
if (SMDrvClear(dev) < 0)
  return(ERR_HARDWARE_OFFLINE);
// **** Hardware Online ?
res = TARCommSync(dev);  // <*> neu
if (!res)
  res = TARPing(dev);
// **** Hardware Info Datenbank abfragen
if (!res)
  res = HwReadInfoVar(dev);
if (!res)
  {
  modul_id = GetInfoValueAsULong(GetInfoByIndex(dev, 0x1000));
  if (DriverInit == TCAN_DRV_EX_INIT)
    modul_desc = &ExCanModulTable[0];
  else
    modul_desc = &CanModulTable[0];
  for (; (hw_id = modul_desc->HwId); modul_desc++)
    {
    if (hw_id == modul_id)
      {
      FreeModulDesc(&dev->ModulDesc);
      dev->ModulDesc = CopyModulDesc(modul_desc);
      break;
      }
    }
  if (!hw_id)
    res = -1;      // Modul unbekannt
  }
// **** Modul Description an Hardware Info Datenbank anpassen
if (!res)
  {
  if ((info = GetInfoByIndex(dev, 0x8001)))  //CAN Features Flags
    {
    value = GetInfoValueAsULong(info);
    dev->ModulDesc->CanFeaturesFlags = value;
    if (value & 0x0004)
      dev->ModulDesc->SupportTxAck = 1;
    else
      dev->ModulDesc->SupportTxAck = 0;
    if (value & 0x0008)
      dev->ModulDesc->SupportCanErrorMsgs = 1;
    else
      dev->ModulDesc->SupportCanErrorMsgs = 0;
    }
  if ((info = GetInfoByIndex(dev, 0x8050)))  // Anzahl Interval Puffer
    dev->ModulDesc->HwTxPufferCount = GetInfoValueAsULong(info);
  if ((info = GetInfoByIndex(dev, 0x8060)))  // Anzahl Filter
    dev->ModulDesc->HwRxFilterCount = GetInfoValueAsULong(info);
  }
if (!res)
  {
  res = TARGetStatus(dev, &tar_status);
  SetTarStatusToDev(dev, tar_status);
  }
// **** FIFOs lschen
if (!res)
  res = TARRxFifoClear(dev, 0);
if (!res)
  res = TARTxFifoClear(dev, 0);
dev->NeedConfig = 1;     
mhs_thread_set_priority(dev->MainThread, priority);
if (!res)
  {
  if (mhs_run_thread(dev->MainThread) < 0)
    res = -1;
  }
if (res < 0)
  { // Port erfolgrich geffnet
  dev->DeviceStatus.DrvStatus = DRV_STATUS_PORT_OPEN;    // Com Port erfolgreich geffnet
#ifdef ENABLE_LOG_SUPPORT
  if (!drv_type)   // RS232
    {
#ifdef __WIN32__
// ****** Windows
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (Port:%d Baudrate:%d): %s", port,
      baud_rate, GetErrorString(res));
#else
// ****** Linux, Mac
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (Device:%s Baudrate:%d): %s", device_name,
      baud_rate, GetErrorString(res));
#endif
    }
  else
    {
    LogPrintf(LOG_ERROR | LOG_MESSAGE, 0, "FEHLER: Open CAN-Device (%s Snr.:%s): %s",
      usb_description, snr, GetErrorString(res));
    }
#endif
  }
else
  { // Device erfolgreich geffnet
#ifdef ENABLE_LOG_SUPPORT
  if (!drv_type)   // RS232
    {
#ifdef __WIN32__
// ****** Windows
    LogPrintf(LOG_MESSAGE, 0, "MESSAGE: CAN-Device erfolgreich geoeffnet (Port:%d Baudrate:%d)", port, baud_rate);
#else
// ****** Linux, Mac
    LogPrintf(LOG_MESSAGE, 0, "MESSAGE: CAN-Device erfolgreich geffnet (Device:%s Baudrate:%d)", device_name,
      baud_rate);
#endif
    }
  else
    {
    LogPrintf(LOG_MESSAGE, 0, "MESSAGE: CAN-Device erfolgreich geffnet (%s Snr.:%s)",
      usb_description, snr);
    }
#endif
  }
if ((!res) && (drv_type))  // kein Fehler, drv_type = USB
  PnPSetDeviceStatus(dev->Index, snr, PNP_DEVICE_ONLINE, 1);

if (DriverInit == TCAN_DRV_STD_INIT)
  (void)GetDevicePnPStatus();
return(res);
}


void CloseCanDevice(TCanDevice *dev)
{
int32_t flag;

flag = 0;
(void)mhs_cmd_execute(dev, MCMD_DEVICE_CLOSE, NULL, 0, 1000);
if (mhs_thread_join_status(dev->MainThread))
  {
  if (mhs_join_thread(dev->MainThread, 5000) < -1) // Timeout 5 Sek.
    flag = 1;
  }
else
  flag = 1;
if (flag)
  {
  dev->DeviceStatus.DrvStatus = DRV_STATUS_INIT;    // Com Port geschlossen
  MainUpdateStatusProc(dev);
  PnPSetDeviceStatus(dev->Index, NULL, PNP_PORT_CLOSE, 1);
  }
ComIoDestroy(&dev->Io);
}


int32_t SetIntervallBox(TCanDevice *dev, uint32_t index, uint16_t cmd, uint32_t time)
{
TMhsObjContext *context;
TObjCanPuffer *puffer;

if (!dev)
  return(-1);
context = dev->TxPufferContext;
if (!dev->ModulDesc)
  return(-1);
if ((index & 0xFFFF) > dev->ModulDesc->HwTxPufferCount)
  return(-1);
if ((puffer = (TObjCanPuffer *)mhs_object_get_by_index(index, context)))
  {
  if (cmd & 0x8000)
    mhs_can_puffer_set_intervall(puffer, time);
  }
return(0);
}


/***************************************************************/
/*  Betriebsart einstellen                                     */
/***************************************************************/
static int32_t ExecuteCanCommand(TCanDevice *dev, uint32_t can_op_mode, uint16_t can_command)
{
TMhsObjContext *context;
int32_t err;
unsigned char cmd, tar_can_op_mode; //, i;
#if (TX_PUFFER_MAX > 0) || (RX_FILTER_MAX > 0)
uint32_t i;
#endif
#if RX_FILTER_MAX > 0
TObjCan *filter;
#endif
#if TX_PUFFER_MAX > 0
TObjCanPuffer *puffer;
#endif

cmd = 0;
switch (can_op_mode)
  {
  case MCMD_CAN_RESET     : {
                            tar_can_op_mode = OP_CAN_RESET;
                            break;
                            }
  case MCMD_CAN_START     : {
                            tar_can_op_mode = OP_CAN_START;
                            break;
                            }
  case MCMD_CAN_START_LOM : {
                            tar_can_op_mode = OP_CAN_START_LOM;
                            break;
                            }
  case MCMD_CAN_START_NO_RETRANS :
                            {
                            tar_can_op_mode = OP_CAN_START_NO_RETRANS;
                            break;
                            }
  case MCMD_CAN_STOP :      {
                            tar_can_op_mode = OP_CAN_STOP;
                            break;
                            }
  default :                 tar_can_op_mode = OP_CAN_NO_CHANGE;
  }
// RxD Fifo lschen
if (can_command & CAN_CMD_RXD_FIFOS_CLEAR)
  {
  mhs_can_fifo_clear_by_index(dev->Index);
  cmd |= 0x20;
  }
// RxD Overrun Flag lschen
else if (can_command & CAN_CMD_RXD_OVERRUN_CLEAR)
  {
  mhs_can_fifo_overrun_clear_by_index(dev->Index);
  cmd |= 0x10;
  }
// TxD Fifo lschen
if (can_command & CAN_CMD_TXD_FIFOS_CLEAR)
  {
  mhs_can_fifo_clear_by_index(dev->Index | INDEX_TXT_FLAG);
  cmd |= 0x80;
  }
// TxD Overrun Flag lschen
else if (can_command & CAN_CMD_TXD_OVERRUN_CLEAR)
  {
  mhs_can_fifo_overrun_clear_by_index(dev->Index | INDEX_TXT_FLAG);
  cmd |= 0x40;
  }
// Alle Hardware Filter lschen
if (can_command & CAN_CMD_HW_FILTER_CLEAR)
  {
#if RX_FILTER_MAX > 0
  context = dev->RxFilterContext;
  //for (i = 1; i <= dev->ModulDesc->HwRxFilterCount; i++)
  for (i = 1; i <= RX_FILTER_MAX; i++)
    {
    if ((filter = (TObjCan *)mhs_object_get_by_index(i, context)))
      {
      filter->Filter.Maske = 0;
      filter->Filter.Code = 0;
      filter->Filter.FilFlags = 0;
      if (i <= dev->ModulDesc->HwRxFilterCount)
        (void)TARSetFilter(dev, i, &filter->Filter);
      }
    }
#endif
  }
// Alle Software Filter lschen
if (can_command & CAN_CMD_SW_FILTER_CLEAR)
  can_sw_filter_remove_all();
if (can_command & CAN_CMD_TXD_PUFFERS_CLEAR)
  {
#if TX_PUFFER_MAX > 0
// **** Intervall Boxen setzen
  context = dev->TxPufferContext;
  for (i = 1; i <= dev->ModulDesc->HwTxPufferCount; i++)
    {
    if ((puffer = (TObjCanPuffer *)mhs_object_get_by_index(i, context)))
      {
      mhs_can_puffer_set_intervall(puffer, 0);
      if ((err = TARCanSetIntervall(dev, i, 0)) < 0)
        return(err);
      }
    }
#endif
  }
return(TARSetCANMode(dev, (unsigned char)(tar_can_op_mode | cmd)));
}


static int32_t set_can_speed(TCanDevice *dev, uint16_t can_speed, uint32_t can_speed_user)
{
if (can_speed)
  return(TARCanSetSpeed(dev, can_speed));
else
  return(TARCanSetBtr(dev, can_speed_user));
}


static void SetCanSpeed1CB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;
uint16_t can_speed;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
if (!(can_speed = obj->Value.U16)) // <*> neu
  return;
(void)set_can_speed(dev, can_speed, 0);
}


static void SetCanSpeed1UserCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;
uint32_t can_speed_user;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
can_speed_user = obj->Value.U32;
(void)mhs_value_set_as_uword("CanSpeed1", 0, 0xFFFF, 0, dev->Context);
(void)set_can_speed(dev, 0, can_speed_user);
}


static void SetCanTxAckEnableCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
if (!dev->ModulDesc)
  return;
if (dev->ModulDesc->SupportTxAck)
  (void)TARCanSetTxAckEnable(dev, obj->Value.U8);
}


static void SetCanSetErrorLogEnableCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
if (!dev->ModulDesc)
  return;
if (dev->ModulDesc->SupportCanErrorMsgs)
  (void)TARCanSetErrorLogEnable(dev, obj->Value.U8);
}


static void SetFilterReadIntervallCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
dev->FilterReadIntervall = obj->Value.U32;
}


static void SetHighPollIntervallCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
dev->HighPollIntervall = obj->Value.U32;
}


static void SetIdlePollIntervallCB(TObjValue *obj, void *user_data)
{
TCanDevice *dev;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
dev->IdlePollIntervall = obj->Value.U32;
}


static void SetIntervallBoxCB(TMhsObj *obj, void *user_data)
{
TCanDevice *dev;
TObjCanPuffer *puffer;
struct TCanMsg msg;
uint32_t idx, intervall;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
if (!dev->ModulDesc)
  return;
puffer = (TObjCanPuffer *)obj;
idx = obj->Index & 0xFFFF;
if (idx > dev->ModulDesc->HwTxPufferCount)
  return;
if (mhs_can_puffer_get(puffer, &msg) >= 0)
  {
  (void)TARTxFifoPut(dev, idx, &msg, 1);
  intervall = puffer->Intervall;
  }
else
  intervall = 0;
(void)TARCanSetIntervall(dev, idx, intervall);
}


static void SetFilterBoxCB(TMhsObj *obj, void *user_data)
{
TCanDevice *dev;
TObjCan *filter;
uint32_t idx;

if ((!obj) || (!user_data))
  return;
dev = (TCanDevice *)user_data;
if (!dev->ModulDesc)
  return;
filter = (TObjCan *)obj;
idx = obj->Index;
if (idx > dev->ModulDesc->HwRxFilterCount)
  return;
(void)TARSetFilter(dev, idx, &filter->Filter);
}


static int32_t ApplyCanSettings(TCanDevice *dev)
{
TMhsObjContext *context, *main_context;
unsigned char tx_ack_enable, can_err_msgs_enable, time_stamp_mode;
uint16_t can_speed;
uint32_t can_speed_user;
int32_t err;
#if (TX_PUFFER_MAX > 0) || (RX_FILTER_MAX > 0)
uint32_t i;
TObjCanPuffer *puffer;
struct TCanMsg msg;
#endif

context = dev->Context;
main_context = can_main_get_context();
can_speed = mhs_value_get_as_uword("CanSpeed1", ACCESS_INIT, context);
can_speed_user = mhs_value_get_as_ulong("CanSpeed1User", ACCESS_INIT, context);
tx_ack_enable = mhs_value_get_as_ubyte("CanTxAckEnable", ACCESS_INIT, context);
can_err_msgs_enable = mhs_value_get_as_ubyte("CanErrorMsgsEnable", ACCESS_INIT, context);
if (mhs_value_get_status("TimeStampMode", context) & MHS_VAL_FLAG_WRITE)
  time_stamp_mode = mhs_value_get_as_ubyte("TimeStampMode", ACCESS_INIT, context);
else
  time_stamp_mode = mhs_value_get_as_ubyte("TimeStampMode", ACCESS_INIT, main_context);
dev->TimeStampMode = time_stamp_mode;
if ((err = TARSetCANMode(dev, OP_CAN_STOP)))  // CAN-Bus stoppen
  return(err);    
if (dev->ModulDesc->SupportHwTimestamp)
  {
  // **** Hardware Time Stamps einstellen
  if (time_stamp_mode >= TIME_STAMP_HW_UNIX)
    i = 1;
  else
    i = 0;
  if ((err = TARSetTimeStamp(dev, (unsigned char)i)) < 0)
    return(err);
  }
if (dev->ModulDesc->SupportTxAck)
  {
  if ((err = TARCanSetTxAckEnable(dev, tx_ack_enable)))
    return(err);
  }
if (dev->ModulDesc->SupportCanErrorMsgs)
  {
  if ((err = TARCanSetErrorLogEnable(dev, can_err_msgs_enable)))
    return(err);
  }
// **** bertragungsgeschwindigkeit einstellen
if ((err = set_can_speed(dev, can_speed, can_speed_user)))
  return(err);
#if RX_FILTER_MAX > 0
// **** Filter einstellen
context = dev->RxFilterContext;
for (i = 1; i <= dev->ModulDesc->HwRxFilterCount; i++)
  {
  if ((puffer = (TObjCanPuffer *)mhs_object_get_by_index(i, context)))
    {
    if ((err = TARSetFilter(dev, i, &((TObjCan *)puffer)->Filter)) < 0)
      return(err);
    }
  }
#endif
#if TX_PUFFER_MAX > 0
// **** Intervall Boxen setzen
context = dev->TxPufferContext;
for (i = 1; i <= dev->ModulDesc->HwTxPufferCount; i++)
  {
  if ((puffer = (TObjCanPuffer *)mhs_object_get_by_index(i | INDEX_TXT_FLAG, context)))
    {
    if (mhs_can_puffer_get_copy(puffer, &msg) > 0)
      {
      if ((err = TARTxFifoPut(dev, i, &msg, 1)) < 0)
        return(err);
      }
    else
      mhs_can_puffer_set_intervall(puffer, 0);
    if ((err = TARCanSetIntervall(dev, i, mhs_can_puffer_get_intervall(puffer))) < 0)
      return(err);
    }
  else
    {
    if ((err = TARCanSetIntervall(dev, i, 0)) < 0)
      return(err);
    }
  }
#endif
if ((err = TARSetCANMode(dev, 0xF0)))   // Rx & Tx FIFO lschen, Rx-OV & Tx-OV lschen
  return(err);
return(0);
}


/***************************************************************/
/*  Main Commandos Event Funktion                              */
/***************************************************************/
static void MainCommandoProc(TCanDevice *dev)
{
int32_t res;
unsigned char time_stamp_mode;
struct TCmdSetModeParam *set_mode_param;
TMhsObjContext *main_context;

res = 0;
main_context = can_main_get_context();
switch (dev->Cmd.Command)
  {
  case MCMD_DEVICE_CLOSE    : {
                              if (mhs_value_get_as_ubyte("AutoStopCan", ACCESS_PORT_OPEN, dev->Context))
                                (void)TARSetCANMode(dev, OP_CAN_STOP);
                              break;
                              }  
  case MCMD_CAN_START       :
  case MCMD_CAN_START_LOM   :
  case MCMD_CAN_START_NO_RETRANS :
  case MCMD_CAN_RESET       :
                              {
                              if (!dev->NeedConfig)
                                {
                                if (mhs_value_get_status("TimeStampMode", dev->Context) & MHS_VAL_FLAG_WRITE)
                                  time_stamp_mode = mhs_value_get_as_ubyte("TimeStampMode", ACCESS_INIT, dev->Context);
                                else
                                  time_stamp_mode = mhs_value_get_as_ubyte("TimeStampMode", ACCESS_INIT, main_context);                                
                                if (dev->TimeStampMode != time_stamp_mode) 
                                  dev->NeedConfig = 1;  
                                }
                              if (dev->NeedConfig)
                                {
                                if (!(res = ApplyCanSettings(dev)))
                                  dev->NeedConfig = 0;
                                else  
                                  break;
                                }
                              }
  case MCMD_CAN_STOP        :
  case MCMD_CAN_ONLY_CMD    : {                              
                              if (dev->Cmd.ParamCount != sizeof(struct TCmdSetModeParam))
                                {
                                res = ERR_PARAM;
                                break;
                                }
                              set_mode_param = (struct TCmdSetModeParam *)dev->Cmd.Param;
                              res = ExecuteCanCommand(dev, dev->Cmd.Command, set_mode_param->CanCommand);
                              dev->StatusTimeStamp = get_tick();     // Status Timeout resetten
                              dev->FilterTimeStamp = get_tick();     // Filter Timeout resetten
                              dev->DeviceStatus.CanStatus = CAN_STATUS_OK;
                              dev->DeviceStatus.FifoStatus = FIFO_OK;
                              if (dev->Cmd.Command != MCMD_CAN_STOP)
                                dev->DeviceStatus.DrvStatus = DRV_STATUS_CAN_RUN;
                              else
                                dev->DeviceStatus.DrvStatus = DRV_STATUS_CAN_OPEN;
                              break;
                              }
  }
mhs_cmd_finish(dev, res);
}


void DevStatusEventSetup(TCanDevice *dev, TMhsEvent *event, uint32_t events)
{
dev->DrvStatusEvent = event;
dev->DrvStatusEvents = events;
}


/***************************************************************/
/*  Treiber Status auf vernderung prfen                      */
/***************************************************************/
static void MainUpdateStatusProc(TCanDevice *dev)
{
uint32_t change;

change = 0;
// Fifo Overrun Status anpassen
//if (CanFifoOverrun(CanInt[0].CanRxDFifo))
//  CanInt[0].DeviceStatus.FifoStatus |= FIFO_SW_OVERRUN;
if (dev->DeviceStatus.DrvStatus < DRV_STATUS_CAN_RUN)
  {
  dev->DeviceStatus.CanStatus = CAN_STATUS_UNBEKANNT;
  dev->DeviceStatus.FifoStatus = FIFO_STATUS_UNBEKANNT;
  }
if (dev->DeviceStatus.DrvStatus != dev->LastDeviceStatus.DrvStatus)
  {
  dev->LastDeviceStatus.DrvStatus = dev->DeviceStatus.DrvStatus;
  change = 1;
  }
if (dev->DeviceStatus.CanStatus != dev->LastDeviceStatus.CanStatus)
  {
  dev->LastDeviceStatus.CanStatus = dev->DeviceStatus.CanStatus;
  change = 1;
  }
if (dev->DeviceStatus.FifoStatus != dev->LastDeviceStatus.FifoStatus)
  {
  dev->LastDeviceStatus.FifoStatus = dev->DeviceStatus.FifoStatus;
  change = 1;
  }
if (change)
  {
  dev->StatusChange = 1;
#ifdef ENABLE_LOG_SUPPORT
  LogPrintDrvStatus(LOG_STATUS, 0, convert_device_idx_to_idx(dev->Index), &dev->DeviceStatus);
#endif
  mhs_event_set(dev->DrvStatusEvent, dev->DrvStatusEvents);
  }
}


/***************************************************************/
/*  Main Thread                                                */
/***************************************************************/
static void MainThreadExecute(TMhsThread *thread)
{
int32_t res;
TCanDevice *dev;
TMhsObjContext *main_context;
uint32_t events, sleep_time, low_poll_intervall, idle_poll_intervall, high_poll_intervall;

dev = (TCanDevice *)thread->Data;
main_context = can_main_get_context();
// ****** Thread starten
if (mhs_value_get_status("LowPollIntervall", dev->Context) & MHS_VAL_FLAG_WRITE)
  low_poll_intervall = mhs_value_get_as_ulong("LowPollIntervall", ACCESS_INIT, dev->Context);
else
  low_poll_intervall = mhs_value_get_as_ulong("LowPollIntervall", ACCESS_INIT, main_context);
dev->StatusTimeStamp = get_tick();         // Status Timeout resetten
dev->FilterTimeStamp = get_tick();         // Filter Timeout resetten
dev->DeviceStatus.DrvStatus = DRV_STATUS_CAN_OPEN;
MainUpdateStatusProc(dev);
// **** Object Event Callbacks verarbeiten
mhs_process_cmd_events(dev->Context);
// ****** Thread Schleife ******
do
  {
  dev->TimeNow = get_tick();
  // **** Events
  events = mhs_event_get((TMhsEvent *)thread, 1);  // Events abfragen u. lschen
  // **** Events verarbeiten
  if (events & MHS_TERMINATE)
    break;
  if (events & MAIN_CMD_EVENT)
    MainCommandoProc(dev);
  // **** Object Event Callbacks verarbeiten
  mhs_process_cmd_events(dev->Context);
#if TX_PUFFER_MAX > 0
  mhs_process_events(dev->TxPufferContext);
  mhs_process_cmd_events(dev->TxPufferContext);
#endif
#if RX_FILTER_MAX > 0
  mhs_process_cmd_events(dev->RxFilterContext);
#endif
  if (dev->DeviceStatus.DrvStatus == DRV_STATUS_CAN_RUN)
    {
    if ((res = MainCommProc(dev)) <= ERR_SM_PORT_NOT_OPEN)
      break;
	  if (res < 0)
      {
      dev->DeviceStatus.DrvStatus = DRV_STATUS_PORT_OPEN;
      PnPSetDeviceStatus(dev->Index, NULL, PNP_DEVICE_OFFLINE, 0);
      }
    else
      {  
      if (dev->IdlePollIntervall != 0xFFFFFFFF)
        idle_poll_intervall = dev->IdlePollIntervall;
      else
        idle_poll_intervall = GlobalVar.IdlePollIntervall;
      if (dev->HighPollIntervall != 0xFFFFFFFF)
        high_poll_intervall = dev->HighPollIntervall;
      else
        high_poll_intervall = GlobalVar.HighPollIntervall;
      
      if (!dev->CommIdleCounter)
        {
        if ((sleep_time = idle_poll_intervall) < high_poll_intervall )
          sleep_time = high_poll_intervall ; 
        }
      else  
        sleep_time = high_poll_intervall ;
      if (sleep_time)
        mhs_sleep_ex((TMhsEvent *)thread, sleep_time);  
      }    
    }
  else
    {
    // Noch keine Verbindug zur Hardware hergestellt
    // Prfen ob Verbindung hergestellt werden kann
    // if ((res = TARPing(dev)) <= ERR_SM_PORT_NOT_OPEN)  // <*> alt
    if ((res = TARCommSync(dev)) <= ERR_SM_PORT_NOT_OPEN)    
      break;
/*    if (dev->DeviceStatus.DrvStatus == DRV_STATUS_PORT_OPEN)
      {
      if (res >= 0)
        PnPSetDeviceStatus(dev->Index, NULL, PNP_DEVICE_ONLINE);
      }
    // Verbindung Hergestellt
    // Prfen ob Verbindung noch besteht
    else if ((dev->DeviceStatus.DrvStatus == DRV_STATUS_DEVICE_FOUND) ||
             (dev->DeviceStatus.DrvStatus == DRV_STATUS_CAN_OPEN))
      {*/
      if (res >= 0)
        {
        dev->DeviceStatus.DrvStatus = DRV_STATUS_CAN_OPEN;
        PnPSetDeviceStatus(dev->Index, NULL, PNP_DEVICE_ONLINE, 0);
        }
      else
        {
        dev->DeviceStatus.DrvStatus = DRV_STATUS_PORT_OPEN;
        PnPSetDeviceStatus(dev->Index, NULL, PNP_DEVICE_OFFLINE, 0);
        }
    dev->CommIdleCounter = 0;
    mhs_sleep_ex((TMhsEvent *)thread, low_poll_intervall);
    }
  MainUpdateStatusProc(dev);
  }
while (thread->Run);
// ****** Thread beenden ******
dev->DeviceStatus.DrvStatus = DRV_STATUS_INIT;    // Com Port geschlossen
MainUpdateStatusProc(dev);
PnPSetDeviceStatus(dev->Index, NULL, PNP_PORT_CLOSE, 0);
}
