#ifndef __TCAN_DRV_H__
#define __TCAN_DRV_H__

#include "global.h"  // Definition von ATTRIBUTE_INTERNAL
#include "com_io.h"
#include "var_obj.h"
#include "mhs_thread.h"
//#include "usb_hlp.h"

#ifdef __cplusplus
  extern "C" {
#endif

#define DEVICE_NAME_MAX_SIZE   255
#define SERIAL_NUMBER_MAX_SIZE 16 
#define DESCRIPTION_MAX_SIZE   64

#define TIME_STAMP_OFF        0
#define TIME_STAMP_SOFT       1
#define TIME_STAMP_HW_UNIX    2
#define TIME_STAMP_HW         3
#define TIME_STAMP_HW_SW_UNIX 4

#define MCMD_DEVICE_CLOSE     1
/*#define MCMD_CAN_OPEN         2
#define MCMD_APPLAY_SETTINGS  3 */
#define MCMD_CAN_STOP         4
#define MCMD_CAN_START        5
#define MCMD_CAN_RESET        6
/*#define MCMD_CAN_SET_SPEED    7
#define MCMD_CAN_SET_FILTER   8
#define MCMD_CAN_TX_SET       9
#define MCMD_CAN_SET_OPT      10
#define MCMD_READ_INFO        11  */
#define MCMD_CAN_ONLY_CMD     12
//#define MCMD_CAN_SET_SPEED_USER 13

#define MCMD_CAN_START_LOM 14
#define MCMD_CAN_START_NO_RETRANS 15
#define MCMD_CAN_SET_TX_ACK_ENABLE 16


#define DEV_LIST_SHOW_TCAN_ONLY 0x01
#define DEV_LIST_SHOW_UNCONNECT 0x02


#pragma pack(push, 1)
struct TModulFeatures  // In tcan_drv.h kopiert
  {
  uint32_t CanClock;
  uint32_t Flags;
  uint32_t CanChannelsCount;
  uint32_t HwRxFilterCount;
  uint32_t HwTxPufferCount;
  };
#pragma pack(pop)  


#pragma pack(push, 1)
struct TCanDevicesList
  {
  uint32_t TCanIdx;
  uint32_t HwId;
  char DeviceName[DEVICE_NAME_MAX_SIZE];
  char SerialNumber[SERIAL_NUMBER_MAX_SIZE];
  char Description[DESCRIPTION_MAX_SIZE];
  struct TModulFeatures ModulFeatures;
  };
#pragma pack(pop)  


#pragma pack(push, 1)
struct TCanDeviceInfo
  {
  uint32_t HwId;
  uint32_t FirmwareVersion;
  uint32_t FirmwareInfo;
  char SerialNumber[SERIAL_NUMBER_MAX_SIZE];
  char Description[DESCRIPTION_MAX_SIZE];
  struct TModulFeatures ModulFeatures;
  };
#pragma pack(pop)


struct TCmdSetModeParam
  {
  uint32_t Index;
  uint16_t CanCommand;
  };


#pragma pack(push, 1)
struct TInfoVar
  {
  uint32_t Key;
  uint32_t Type; 
  uint32_t Size;
  char Data[255];
  };
#pragma pack(pop)


struct TInfoVarList
  {
  struct TInfoVarList *Next;
  unsigned char Size;
  unsigned char Type;
  uint16_t Index;
  const char *IndexString;
  char *Data;
  };

struct TCanModulDesc
  {
  uint32_t HwId;
  char *Description;
  uint32_t BaudRate;
  uint32_t UsbDeviceType;
  uint32_t StatusSize;
  uint32_t SupportHwTimestamp;
  uint32_t SupportTxAck;
  uint32_t SupportCanErrorMsgs;
  uint32_t CanFeaturesFlags;
  uint32_t HwTxFifoSize;
  uint32_t HwTxFifoLimit;
  uint32_t HwRxFilterCount;
  uint32_t HwTxPufferCount;
  uint32_t CanClock;
  };

#define SM_DRV_PARAMETER_COUNT 4096

struct TSMDrvParameter
  {
  unsigned char Data[SM_DRV_PARAMETER_COUNT];
  uint32_t Count;
  };

typedef struct _TMhsCmd TMhsCmd;

struct _TMhsCmd
  {
#ifdef __WIN32__  
  HANDLE Sem;
  HANDLE WaitSem;
#else  
  pthread_mutex_t Mutex;
  pthread_mutex_t WaitMutex;
  pthread_cond_t Cond;
#endif  
  uint32_t Command;
  uint32_t ParamCount;
  char *Param;
  uint32_t Result;
  };

typedef struct _TCanDevice TCanDevice;

struct _TCanDevice
  {
  // **** Device Index
  uint32_t Index;
  // **** Reference Counter
  volatile uint32_t RefCount;
  // **** Context
  TMhsObjContext *IoContext;
  TMhsObjContext *Context;
  TMhsObjContext *TxPufferContext;
  TMhsObjContext *RxFilterContext;
  // **** Kommando
  TMhsCmd Cmd;
  // **** Main Thread
  TMhsThread *MainThread;
  struct TIo *Io;
  // **** Modul Beschreibung  
  struct TCanModulDesc *ModulDesc;
  struct TInfoVarList *InfoVars;
  char *DrvInfoHwStr;
  // **** Uebergabepuffer fr Callback-Funktion
  struct TCanMsg *CanRxDBuffer;
  uint32_t CanRxDBufferSize;

  // **** Variablen
  unsigned char NeedConfig;
  uint32_t TimeNow;
  uint32_t StatusTimeStamp;
  uint32_t FilterTimeStamp;
  uint32_t HwTxDFifoCount;
  uint32_t ReadStatusFlag;
  struct TCanMsg TxCanBuffer[HW_TX_FIFO_MAX];
  uint32_t CommIdleCounter;
  // **** Konfigurationsvariablen
  unsigned char TimeStampMode;
  uint32_t FilterReadIntervall;
  uint32_t HighPollIntervall;
  uint32_t IdlePollIntervall;  
  // **** Device Status
  struct TDeviceStatus DeviceStatus;
  struct TDeviceStatus LastDeviceStatus;
  int32_t StatusChange;
  TMhsEvent *DrvStatusEvent;
  uint32_t DrvStatusEvents;
  // **** SM
  unsigned char TxDDataPuffer[SM_DRV_PARAMETER_COUNT+4];
  unsigned char Kommando;
  struct TSMDrvParameter TxDParameter;  // Sende Parameter
  struct TSMDrvParameter RxDParameter;  // Empfangene Parameter
  unsigned char ACKByte;                // Empfangenes ACK Byte

  //int32_t Status;
  };


extern const struct TCanModulDesc CanModulTable[] ATTRIBUTE_INTERNAL;
extern const struct TCanModulDesc ExCanModulTable[] ATTRIBUTE_INTERNAL; 

TCanDevice *CreateCanDevice(void) ATTRIBUTE_INTERNAL;
void DestroyCanDevice(TCanDevice **device) ATTRIBUTE_INTERNAL;
int32_t GetDeviceInfo(TCanDevice *dev, struct TCanDeviceInfo *device_info) ATTRIBUTE_INTERNAL;
int32_t OpenCanDevice(TCanDevice *dev) ATTRIBUTE_INTERNAL;
void CloseCanDevice(TCanDevice *dev) ATTRIBUTE_INTERNAL;
int32_t SetIntervallBox(TCanDevice *dev, uint32_t index, uint16_t cmd, uint32_t time) ATTRIBUTE_INTERNAL;

void DevStatusEventSetup(TCanDevice *dev, TMhsEvent *event, uint32_t events) ATTRIBUTE_INTERNAL;

#ifdef __cplusplus
  }
#endif

#endif
