#!/usr/bin/python
# -*- coding: utf-8 -*-

#       TinyCanMon.py - Python Tiny-CAN Monitor
#  Copyright (C) 2015 MHS-Elektronik GmbH & Co. KG 

VERSION = \
"""
TinyCanMon V0.01, 12.05.2015 (LGPL)
K.Demlehner (klaus@mhs-elektronik.de)
"""
import sys
sys.path.append('../.')
#from gi.repository import Gtk, GObject, GLib
import gtk as Gtk
import gobject as GObject
import os
import time
import mhsTinyCanDriver as CanDriver
from baseOptionParser import BaseOptionParser


class TinyCanMon:
    def __init__ ( self ):        
        # read ui File
        self.ui = Gtk.Builder()
        self.ui.add_from_file("TinyCanMon.ui")
        self.MainFrame = self.ui.get_object("CanMonWin") 
        # get widget references
        self.StatusBar = self.ui.get_object("StatusBar")
        self.SBContext = self.StatusBar.get_context_id("common context id")
        self.BnCC = self.ui.get_object("BnCreateClose")

        self.BnScroll = self.ui.get_object("BnScroll")
        self.BnScroll.set_active(0)
        self.Scroll = 0
        
        self.EnID = self.ui.get_object("EnID")
        self.EnDLC = self.ui.get_object("EnDLC")
        self.Ext = self.ui.get_object("Ext")
        self.EnD0 = self.ui.get_object("EnD0")
        self.EnD1 = self.ui.get_object("EnD1")
        self.EnD2 = self.ui.get_object("EnD2")
        self.EnD3 = self.ui.get_object("EnD3")
        self.EnD4 = self.ui.get_object("EnD4")
        self.EnD5 = self.ui.get_object("EnD5")
        self.EnD6 = self.ui.get_object("EnD6")
        self.EnD7 = self.ui.get_object("EnD7")
        self.EnD7 = self.ui.get_object("EnD7")
        
        self.CBBaud = self.ui.get_object("CBBaud")
        self.SBCanStatus = self.ui.get_object("SBCanStatus")
        self.SBCanContext = self.SBCanStatus.get_context_id("can context id")
                
        # TreeView MVC Initialization
        self.TV = self.ui.get_object("TreeView")
        # Create a Model for the Data to store
        self.model = Gtk.TreeStore(            
            GObject.TYPE_STRING,
            GObject.TYPE_STRING,
            GObject.TYPE_STRING,
            GObject.TYPE_UINT,
            GObject.TYPE_STRING)
        self.TV.set_model(self.model)
        
        self.Renderer = Gtk.CellRendererText()
        column0 = Gtk.TreeViewColumn(" Dir ", self.Renderer, text=0)
        self.TV.append_column(column0)
        column1 = Gtk.TreeViewColumn(" Ext/Std   ", self.Renderer, text=1)
        self.TV.append_column(column1)
        column2 = Gtk.TreeViewColumn(" ID                ", self.Renderer, text=2)
        self.TV.append_column(column2)
        column3 = Gtk.TreeViewColumn(" DLC ", self.Renderer, text=3)
        self.TV.append_column(column3)
        column4 = Gtk.TreeViewColumn(" DATA (HEX)", self.Renderer, text=4)
        self.TV.append_column(column4)
        
        # can state
        self.CanStarted = 0 


    def show(self):
        self.ui.connect_signals (
        {
         "OnExitApp" : self.OnExitApp,
         "OnBnCreateClose" : self.OnBnCreateClose,
         "OnBnScroll" : self.OnBnScroll,        
         "OnMSend" : self.OnMSend,
         "OnBnCanReset" : self.OnBnCanReset,
         "OnBnInfo" : self.OnBnInfo,
         "OnBnClear" : self.OnBnClear,
         }
        )
        self.CanText("CAN stopped")
        # create the driver
        self.TinyCan = CanDriver.MhsTinyCanDriver()
        self.TinyCan.CanSetUpEvents(PnPEventCallbackfunc=PnPEventCallback, StatusEventCallbackfunc=StatusEventCallback, RxEventCallbackfunc=RxEventCallback)
        Gtk.gdk.threads_enter()
        Gtk.main()
        Gtk.gdk.threads_leave()
        # shutdown
        self.TinyCan.CanSetEvents(0)
        time.sleep(0.5)
        self.TinyCan.CanDownDriver()
        self.TinyCan.so = None
  
  
    def OnBnInfo(self, *args):
        info_ui = Gtk.Builder()
        info_ui.add_from_file("info.ui")
        dialog = info_ui.get_object("InfoDialog");
        response = dialog.run()
        dialog.hide()
        return
  
    
    # all the things to do before exit this application
    def ExitApp(self):           
        Gtk.main_quit()
            
    # remove the old statusbar message and add a new one
    def Text(self, Message ):
        self.StatusBar.pop(self.SBCanContext)
        self.StatusBar.push(self.SBCanContext, Message)
        

    def CanText(self, Message):
        self.SBCanStatus.pop(self.SBContext)
        self.SBCanStatus.push(self.SBContext, Message)
        
        
    # Event handler for the frame window       
    def OnExitApp(self, event, widget_object):
        self.ExitApp()
        return

    def OnBnScroll(self, *args):
        if self.Scroll == 0:
            self.Scroll = 1
        else:
            self.Scroll = 0
        self.BnScroll.set_active(self.Scroll)
        return
              
                           
    def OnBnCanReset(self, *args):
        self.TinyCan.resetCanBus()
        return
        
    def OnBnCreateClose(self, *args):    
        if self.BnCC.get_active():
            self.BnCC.set_label('Disconnect');
            bitrate = self.TinyCan.canBitrates[self.CBBaud.get_active()]
            err = self.TinyCan.OpenComplete(canSpeed=bitrate)
            if err < 0:
                self.CanText('Device Error')
                self.Text(self.TinyCan.FormatError(err, 'Fehler:'))
            else:
                self.CanText('Device Open')
        else:
            self.BnCC.set_label('Connect');
            self.TinyCan.CanDeviceClose();
            self.CanText('Device Close')
            self.Text('')                        
        return


    def OnBnClear(self, *args):
        self.model.clear()
        return

    # be aware that any changes in the definition above must also be
    # put in the following definition
    def OnMSend(self, *args):
        # check the values if valid
        ID = 0
        try:
            ID = int (self.EnID.get_text(), 16 )
        except Exception, ex:
            self.Text ( "Invalid ID" )
            return
        DLC = 0
        try:
            DLC = int(self.EnDLC.get_text(), 10)
        except Exception, ex:
            self.Text ( "Invalid DLC" )
            return        
        if DLC > 8 or DLC < 0:
            self.Text ( "Invalid DLC" )
            return        
        if self.Ext.get_active():
            EFF = 1
        else:
            EFF = 0;    
        D = [0, 0, 0, 0, 0, 0, 0, 0]
        try:
            D[0] = int(self.EnD0.get_text(), 16)
        except Exception, ex:
            D[0] = 0
        try:
            D[1] = int(self.EnD1.get_text(), 16)
        except Exception, ex:
            D[1] = 0
        try:
            D[2] = int(self.EnD2.get_text(), 16)
        except Exception, ex:
            D[2] = 0
        try:
            D[3] = int(self.EnD3.get_text(), 16)
        except Exception, ex:
            D[3] = 0
        try:
            D[4] = int(self.EnD4.get_text(), 16)
        except Exception, ex: 
            D[4] = 0
        try:
            D[5] = int(self.EnD5.get_text(), 16)
        except Exception, ex:
            D[5] = 0
        try:
            D[6] = int(self.EnD6.get_text(), 16)
        except Exception, ex:
            D[6] = 0
        try:
            D[7] = int(self.EnD7.get_text(), 16)
        except Exception, ex:
            D[7] = 0
                      
        self.TinyCan.TransmitData(0, msgId = ID, msgLen = DLC, msgData = D[0:], eff = EFF)
        return
            

    def FormatMessages(self, messages=None):                  
        formatedMessages = []
        if messages:
            for message in messages:
                if message.Flags.FlagBits.RTR and message.Flags.FlagBits.EFF:
                    frame_format = 'EFF/RTR'
                else: 
                    if message.Flags.FlagBits.EFF:
                        frame_format = 'EFF    '
                    else:
                        if message.Flags.FlagBits.RTR:
                            frame_format = 'STD/RTR'
                        else:
                            frame_format = 'STD    '
                if message.Flags.FlagBits.TxD:
                    dir = 'TX'
                else:
                    dir = 'RX'                    
                dlc = message.Flags.FlagBits.DLC;
                if message.Flags.FlagBits.EFF:
                    id_str = '%08X' % message.Id 
                else:    
                    id_str = '%03X' % message.Id
                msg_data = ""
                if not message.Flags.FlagBits.RTR:                                    
                    for n in range(dlc):
                        msg_data = msg_data + '%02X ' % message.Data[n]                    
                row = self.model.append(None)
                self.model.set_value(row, 0, dir)
                self.model.set_value(row, 1, frame_format)                
                self.model.set_value(row, 2, id_str)                  
                self.model.set_value(row, 3, dlc)
                self.model.set_value(row, 4, msg_data)
                if self.Scroll:
                    path = self.model.get_path(row)
                    self.TV.scroll_to_cell (path, None, False, 0.0, 0.0)
        return                 

# --------------------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------

    def ReadAndPrintMessages(self):
        res = self.TinyCan.CanReceive(count = 500)
        if res[0] > 0:                 
            self.FormatMessages(res[1])              
        else:
            if res < 0:
                self.Text(self.TinyCan.FormatError(res, 'CanReceive'))
                return -1
        return 0 

# --------------------------------------------------------------------------------------------------------
# ----------------------- Event Callbacks ----------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------

def PnPEventCallback(index, status):
    """
    Plug & Play Callback
    status: simple Flag, 0 = disconnect, 1 = connect  
    """    
    Gtk.gdk.threads_enter()
    if status:            
        CanMon.CanText('Device Connected')           
    else:
        CanMon.CanText('Device Disconnected')
        CanMon.BnCC.set_active(0)
    Gtk.gdk.threads_leave()    
    return


def StatusEventCallback(index, deviceStatusPointer):
    """
    Status Callback
    @param deviceStatusPointer: Pointer to Device Status  
    """
    Gtk.gdk.threads_enter()
    deviceStatus = deviceStatusPointer.contents
    CanMon.Text(CanMon.TinyCan.FormatCanDeviceStatus(deviceStatus.DrvStatus,deviceStatus.CanStatus,deviceStatus.FifoStatus))
    Gtk.gdk.threads_leave()
    return


def RxEventCallback(index, DummyPointer, count):
    """
    CAN Receive Callback 
    DummyPointer: is set to NULL
    count: Number of Messages
    """     
    Gtk.gdk.threads_enter()   
    CanMon.ReadAndPrintMessages()
    Gtk.gdk.threads_leave()
    return

# --------------------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------
# --------------------------------------------------------------------------------------------------------
Gtk.gdk.threads_init()
CanMon = TinyCanMon()
CanMon.show()



